<?php
/**
 * Controller for authentication functionality (login, register).
 * 
 * @version 0.8.0
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage controllers
 * @uses AuthModel UsersModel
 */
class Auth extends SM_Controller {
	public function Auth() {
		parent::SM_Controller();
		$this->load->model(Array('AuthModel','UsersModel'));
	}
	
	/**
	 * Default controller, redirects appropriately.
	 */
	public function index() {
		if($this->AuthModel->getCurrentUser() != "") { redirect('/dashboard/'); } 
		else { redirect('/auth/register/'); }
	}
	
	/**
	 * Presents registration.
	 * @uses recaptcha
	 */
	public function register() {
		$this->load->helper('recaptcha');
		$this->data("errors", $this->_registerCheck($_POST));
		$this->style(Array('auth.css'));
		$this->contextual(Array('logincallout'));		
		$this->view("authDisplay", "auth/register_view");
		$this->display("Registration", "auth_view");
	}
	
	/**
	 * Presents a stand alone login page.
	 */
	function login() {
		$this->style(Array('auth.css'));
		$this->view("authDisplay", "auth/login_view");
		$this->display("Login", "auth_view");
	}
	
	/**
	 * Logs a user in if they pass valid credentials.
	 * @param $username The username of the user.
	 * @param $password The user's password
	 */
	public function login_post($username=null, $password=null) {
		if($username == null) { $username = $this->input->post('user_name'); }
		if($password == null) { $password = $this->input->post('password'); }
		if($this->AuthModel->loginCurrentUser($username, $password) == "true") {
			if(strstr($_SERVER['HTTP_REFERER'], "auth/register") || strstr($_SERVER['HTTP_REFERER'], "home") || $_SERVER['HTTP_REFERER'] == base_url()) { redirect('dashboard'); }  
			else {redirect($_SERVER['HTTP_REFERER']); }
		} 
		else { $this->forgotpassword("That login didn't work, did you forget your password?"); }
	}
	
	/**
	 * Logs out the current user.
	 */
	public function logout() { $this->AuthModel->logoutCurrentUser(); redirect(''); }
	
	/**
	 * Handles forgotten passwords
	 * @param $error string Any errors from previous submissions
	 */
	public function forgotpassword($error = '') {
		if($this->AuthModel->checkUserLogin()) {
			redirect('dashboard');
		}
		else {
			if($this->input->post('email') != '') {
				$email = $this->input->post('email');
				$user = $this->UsersModel->getUserByEmail($email);
				$this->AuthModel->resetForgottenPassword($email, $user->name);
				redirect('');
			}
			$this->data("errors", $error);
			$this->style(Array('auth.css'));
			$this->view("authDisplay", "auth/forgot_view");
			$this->display("Forgot Password", "auth_view");	
		}	
	}
	
	/**
	 * Changes a users password.
	 */		
	public function changepassword() {
		$password = json_decode($this->input->post("data"));		
		$this->output->set_output($this->AuthModel->changeCurrentUserPassword($password->old, $password->change));		
	}
	
	/**
	 * Handles registration checking, creates users or produces errors.
	 * @param $form the form (post data ) containing necessary registration information.
	 * @return string The errors in registration ("" if nothing).
	 */
	private function _registerCheck($form) {
		$errors = "";
		if(isset($form['submitted'])) {
			if(isset($form['reg_user_name']) 
			&& isset($form['reg_password']) 
			&& isset($form['confirmpassword']) 
			&& isset($form['email'])) 
			{
				$check = $this->_registryEntryCheck($form['reg_user_name'], 
													$form['reg_password'], 
													$form['confirmpassword'], 
													$form['email']);
				if($check != "") { $errors = $check;} 
				else {
					$resp = null;
					if(isset($form['recaptcha_challenge_field']) 
					&& isset($form['recaptcha_response_field'])) 
					{
						$privatekey = "6Ld8BwkAAAAAADz-pFe_twgSmK-pJ8wY4CJlLCFF";
						$resp = recaptcha_check_answer ($privatekey, 
														$_SERVER["REMOTE_ADDR"], 
														$form['recaptcha_challenge_field'], 
														$form['recaptcha_response_field']);
					} // Recaptcha result
					else { $resp->is_valid = true; }
					if (!$resp->is_valid) {
						if($errors != "") { $errors .= " & "; }
						$errors .= "You missed the captcha. Try again and be precise - capitals, punctuation and spacing count."; 
					} // Check if captcha is entered
					else {
						if(!(isset($form['contact']))) { $form['contact'] = 'FALSE';}
						$this->AuthModel->createUser($form['reg_user_name'], $form['reg_password'], $form['email'], $form['description'], $form['contact']);
						$this->AuthModel->loginCurrentUser($form['reg_user_name'], $form['reg_password']);
						redirect('/dashboard/');
					}
				}
			} // Check if form is complete
			else {
				if($errors != "") { $errors .= " & "; }
				$errors .= "Please complete the entire Registration Form.";
			}
		} // Form was submitted
		return $errors;
	}
	
	/**
	 * Determines if a username is already registered, passwords match, and email is unique and valid.
	 * @param $user string The username.
	 * @param $pass string The password
	 * @param $confpass The confirmation of the password
	 * @param $email The user email
	 * @return string The errors in registration ("" if nothing).
	 */
	private function _registryEntryCheck($user, $pass, $confpass, $email) {
		$errors = "";
		if(!($this->UsersModel->checkUserName($user))) {
			if($errors != "") { $errors .= " & "; }
			$errors .= "That username is unavailable, choose another one.";
		}
		if(!($this->UsersModel->checkUserMail($email))) {
			if($errors != "") { $errors .= " & "; }
			$errors .= "That email address is unavailable, choose another one.";
		}
		if(!($pass == $confpass)) {
			if($errors != "") { $errors .= " & "; } 
			$errors .= "Make sure your passwords match.";
		}
		if (!(preg_match("/^[-_a-z0-9\'+*$^&%=~!?{}]++(?:\.[-_a-z0-9\'+*$^&%=~!?{}]+)*+@(?:(?![-.])[-a-z0-9.]+(?<![-.])\.[a-z]{2,6}|\d{1,3}(?:\.\d{1,3}){3})(?::\d++)?$/iD", $email))) {
			if($errors != "") { $errors .= " & "; }
			$errors .= "Make sure you enter a valid email address.";
		}
		return $errors;
	}
} // End Auth
