<?php

class Collections extends SM_Controller {
	
	//Sets the number of active objects on the map/list for view()
	private $objectsPerPage = 50;
	
	//Sets the allowed types and operators for filtering
	private $validTypes = array('*', 'name', 'textual', 'origin', 'weight', 'type', 'emissions', 'description');
	private $validOperators = array('>', '>=', '=', '<', '<=', 'ILIKE', 'NOT_ILIKE');
	
	public function Collections() {
		parent::SM_Controller();
		$this->load->model('AuthModel');
		$this->load->model('CollectionsModel', 'cm');
		$this->load->model('ObjectsModel', 'om');
		$this->load->model('UsersModel', 'um');
		$this->load->model('GroupsModel', 'gm');
		$this->data("current", "objects");		
	}
	
	/**
	* Method used to browse all Sourcemap Collections
	*/
	public function index() {
		$this->load->helper(Array('time','text','string'));
		$this->load->library('pagination');
		
		$pagerange = $this->uri->segment(2);
		$pageper = 10;
		$collections = $this->cm->getCollectionsInRange($pageper, $pagerange);
		$pagetotal = count($this->cm->getCollectionsInRange(null, null));

		$this->data('collections', $collections);
		$this->paginate($pageper, site_url() . '/collections/', $pagetotal, 2);
		$this->view('collectionsList', 'collections/collectionslist_view');
		$this->style(Array('collections.css'));
		$this->script(Array('collections.js'));
		$this->contextual(Array('collectioncreate'));
		
		$this->display("Collections", "collections_view");
	}
	
	/**
	* Admin-only method that confirms the creation of a collection and gives a link to it.
	*/
	public function confirmCreate() {
		if(!($this->AuthModel->checkUserLogin()) || !($this->AuthModel->getCurrentUserRole() == "admin")) {
			redirect('/auth/register/');  
		}
		else {
			//Get form variables
			$collectionName = $this->input->post('name');
			$collectionDesc = $this->input->post('description');
			$ownerType = $this->input->post('ownertype');
			$ownerID = $this->input->post('id');
			$objectField = $this->input->post('field');
			$objectRelation = $this->input->post('relation');
			$objectValue = $this->input->post('value');
			//This collection's criterion:
			$criterion = "";
			
			//Deals with the case in which a user or group's 
			//name is entered, rather than an ID
			if(intval($ownerID) == 0 && $ownerType != '*') {
				if($ownerType == 'user') {
					$ownerID = $this->um->getUserId($ownerID);
				}
				else { //It's owned by a group
					$ownerID = $this->gm->get_group_id($ownerID);
				}
				if($ownerID == 0) {//User by that name doesn't exist
					$this->output->set_output("The specified user or group doesn't exist.");
					return;
				}
			}
			//Build the criterion
			if($ownerType == '*' || $ownerID == '*') {
				$criterion .= "Ownership:*; ";
			} else {
				$criterion .= "Ownership:$ownerType $ownerID; ";
			}
			if($objectField == '*' || $objectValue == '*') {
				$criterion .= "Match:*";
			}
			else {
				$criterion .= "Match:$objectField $objectRelation $objectValue";				
			}

			
			//Create the collection and give confirmation to user
			$collectionID = $this->cm->createCollection($collectionName, $collectionDesc, $criterion);
			$collection = $this->cm->getCollectionByID($collectionID);
			$slug = $collection->slug;
			$msg = "<a href='".site_url()."collection/".$slug."'>"; 
			$msg .= "Collection with ID ".strval($collectionID)."</a> created.<br/>";
			$msg .= "<a href='".site_url()."collections/create'>Back to collections creation</a><br/>";
			$msg .= "<a href='".site_url()."collections'>Back to collections browser</a><br/>";
			$this->output->set_output($msg);
		}
	}
	
	/**
	* Admin-only method to create a new collection.
	*/
	public function create() {
		if(!$this->AuthModel->checkUserLogin() || !($this->AuthModel->getCurrentUserRole() == "admin")) {
			redirect('/auth/register/');  
		}
		else {
			//Grab the columns out of objects_complex for our drop-down menu
			$fields = array();
			$object = array_pop($this->om->getObjects(1));
			foreach ($object as $key => $value) {
				$fields[] = $key;
			}
			$this->data('fields', $fields);
			$this->style(Array('collections.css'));
			$this->contextual(Array('quickticket'));
			$this->display("Create a Collection", "collections/collectionscreate_view");
		}
	}
	
	/**
	* Method used to view an individual collection.
	*/
	public function collection() {
		$this->load->helper(Array('time','text','string','googlemaps', 'locale'));
		$this->style(Array('collections.css', 'objectlist.css', 'objects.css', 'visualizations/objectslist.css'));
		
		//Get the collection and initialize the objects		
		$slug = $this->uri->segment(2);
		$objects = array();
		if($slug) {
			$collection = $this->cm->getCollectionBySlug($slug);
			if($collection != NULL) {
				//Objects that will be displayed in green
				$objects = $this->cm->getObjectsByStatement($collection->criterion, $this->objectsPerPage, 0);
				//Objects that will be displayed in gray
				$otherObjects = $this->cm->getObjectsNotInRange($collection->criterion, $this->objectsPerPage, 0);
			}
			else { //No collection matching the specified ID
				show_404();
			}
		}
		else { //No ID specified
			show_404();
		}
		$collection->objectCount = $this->cm->getObjectCount($collection->id);
		
		$this->data('collection', $collection);
		$this->data('objectsArray', $objects);
		$this->view("mapDisplay", "map/collectionsmap_view");
		$this->view("objectsDisplay", "visualizations/objectslist");		
		$this->contextual(Array('collectiondesc', 'devblog'));
		if($this->config->item('tileprovider') == "cloudmade") {
			$this->script(Array("http://www.google.com/jsapi?autoload=%7B%22modules%22%3A%5B%7B%22name%22%3A%22maps%22%2C%22version%22%3A%222%22%7D%2C%7B%22name%22%3A%22earth%22%2C%22version%22%3A%221%22%7D%5D%7D&key=".getGoogleMapsKey(base_url()), base_url()."assets/scripts/OpenLayers-2.8/OpenLayers.js", base_url()."assets/scripts/OpenLayers-2.8/cloudmade.js"), true);
		} else { 
			$this->script(Array("http://www.google.com/jsapi?autoload=%7B%22modules%22%3A%5B%7B%22name%22%3A%22maps%22%2C%22version%22%3A%222%22%7D%2C%7B%22name%22%3A%22earth%22%2C%22version%22%3A%221%22%7D%5D%7D&key=".getGoogleMapsKey(base_url()), base_url()."assets/scripts/OpenLayers-2.8/OpenLayers.js"), true);		
		}
		$this->script(Array('collections.js', 'object.js', 'openlayers_multimap.js'));
		$this->scriptvar(Array($this->om->generateJSONObjectArray("objectsConfig", $objects), 
							$this->om->generateJSONObjectArray("otherObjects", $otherObjects), 
							"var collectionID='".$collection->id."';",
			 				"Sourcemap.tileprovider = '".$this->config->item('tileprovider')."';",
			 				"Sourcemap.geocodeprovider = '".$this->config->item('geocodeprovider')."';"));
		$this->display(stripslashes($collection->name), "collections/collectionsdisplay_view");		
	}
	
	
	/**
	* Called by ajax to filter a collections map
	*/
	public function filterMap() {		
		$type = $this->input->post('type');
		$operator = $this->input->post('operator');
		$keyword = $this->input->post('keyword');
		$id = $this->input->post('id');
		
		$objects = array();
		
		if(trim($keyword) == "") {
			$objects[0] = "Invalid Input"; //User forgot to put in a search term
		}		
		else if(!in_array($type, $this->validTypes) || !in_array($operator, $this->validOperators) || 
			stristr($keyword, ";") != false || stristr($keyword, "\\") != false || stristr($keyword, "--") != false) {
			
			$objects[0] = "Invalid Input"; //Probably an attempt at SQL Injection. Consider logging this.
		} else {
			$keywords = explode(" ", $keyword);
			$collection = $this->cm->getCollectionByID($id);
			$newCriterion = $collection->criterion;
			if(is_array($keywords)) {
				foreach($keywords as $keyword) {
					$newCriterion.= " ".$type." ".$operator." ".$keyword;						
				}
			} else{
				$newCriterion.= " ".$type." ".$operator." ".$keywords;						
			}
			$newObjects = $this->cm->getObjectsByStatement($newCriterion);
			
			foreach($newObjects as $object) {
				$objects[] = $object;
			}
		}
		$this->output->set_output(json_encode($objects));
	}
	
	/**
	* Called by ajax to filter an object listing in the collections map
	*/
	public function filterList() {
		$type = $this->input->post('type');
		$operator = $this->input->post('operator');
		$keyword = $this->input->post('keyword');
		$id = $this->input->post('id');
		$offset = $this->input->post('offset');
			
		if(trim($keyword) == "") {
			//User forgot to put in a search term
		}		
		else if(!in_array($type, $this->validTypes) || !in_array($operator, $this->validOperators) || 
			stristr($keyword, ";") != false || stristr($keyword, "\\") != false) {	
			//Probably an attempt at SQL Injection. Consider logging this.
		} else {
			$keywords = explode(" ", $keyword);
			$collection = $this->cm->getCollectionByID($id);
			$newCriterion = $collection->criterion;
			if(is_array($keywords)) {
				foreach($keywords as $keyword) {
					$newCriterion.= " ".$type." ".$operator." ".$keyword;							
				}
			} else{			
				$newCriterion.= " ".$type." ".$operator." ".$keywords;										
			}
			if($offset >= 0) {
				$newObjects = $this->cm->getObjectsByStatement($newCriterion, $this->objectsPerPage, $offset);
			} else {
				$newObjects = $this->cm->getObjectsByStatement($newCriterion);
			}
			$this->data("objectsArray", $newObjects);
			$this->display("Object List", "visualizations/objectslist");
		}
	}
	
	
	/**
	* Called by ajax to get the next page worth of objects in a collection
	*/
	public function getNextPage() {
		$oldOffset = $this->input->post('offset');
		$id = $this->input->post('id');
		
		$collection = $this->cm->getCollectionByID($id);
		$objectCount = $this->cm->getObjectCount($id);
		$lastIndex = $objectCount - 1;
		if($objectCount == 0 || $lastIndex < $oldOffset) { return; } //Something went horribly wrong
		
		$newOffset = $oldOffset + $this->objectsPerPage;
		if($newOffset >= $lastIndex) {
			$newOffset = 0;
		}
		$objects = $this->cm->getObjectsByStatement($collection->criterion, $this->objectsPerPage, $newOffset);
		$otherObjects = $this->cm->getObjectsNotInRange($collection->criterion, $this->objectsPerPage, $newOffset);
		$this->output->set_output($this->om->generateJSONObjectArray('objectsConfig', $objects, true) . 
			 $this->om->generateJSONObjectArray('otherObjects', $otherObjects, true) . 
			 "offset = $newOffset;");
	}
	
	
	/**
	* Called by ajax to get the previous page worth of objects in a collection
	*/
	public function getPrevPage() {
		$oldOffset = $this->input->post('offset');
		$id = $this->input->post('id');
		$collection = $this->cm->getCollectionByID($id);
		$objectCount = $this->cm->getObjectCount($id);
		$lastIndex = $objectCount - 1;
		if($objectCount == 0 || $lastIndex < $oldOffset || $this->objectsPerPage > $lastIndex) { return; } //Something went horribly wrong
		
		$newOffset = $oldOffset - $this->objectsPerPage;
		if($newOffset < 0) {
			$newOffset = $lastIndex - ($lastIndex % $this->objectsPerPage);
		}
		$objects = $this->cm->getObjectsByStatement($collection->criterion, $this->objectsPerPage, $newOffset);
		$otherObjects = $this->cm->getObjectsNotInRange($collection->criterion, $this->objectsPerPage, $newOffset);
		$this->output->set_output($this->om->generateJSONObjectArray('objectsConfig', $objects, true) . 
			 $this->om->generateJSONObjectArray('otherObjects', $otherObjects, true) . 
			 "offset = $newOffset;");
	}
}
?>