<?php
/**
 * Controller for groups.
 *
 * @version 0.8.0
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage controllers
 * @uses AuthModel GroupsModel ObjectsModel UsersModel string
 */
class Groups extends SM_Controller {	
	function Groups() {
		parent::SM_Controller();
		$this->load->model(Array('GroupsModel', 'AuthModel', 'UsersModel', 'ObjectsModel'));
		$this->load->helper('string');		
		$this->data("current", "objects");		
	}
	
	/**
	 * Default controller, loads a listing of groups
	 */
	public function index() {			  
		$range = $this->uri->segment(2);
		$pageper = 30;
		$pagetotal = $this->GroupsModel->getGroupsCount();
		$this->paginate($pageper,site_url()."groups/",$pagetotal,2);

		$this->data("groupsArray", $this->GroupsModel->getGroups($pageper, $range));
		if($range <= 1) { $this->data('infomessage', true); }

		$this->view("groupsDisplay", "groups/groupslist_view");
		$this->style(Array('groups.css'));
		$this->display("Groups", "groups_view");
	}		
	
	/**
	 * Loads the page with the group name or slug
	 * @param $slug string the group to display
	 */	
	public function single($slug=null){	 
		$group = $this->GroupsModel->getGroupBySlug($slug);
		if($group == null) {show_404(); }

		$this->data("editable", $this->GroupsModel->checkIsAdmin($this->AuthModel->getCurrentUserID(), $group->id));
		$this->data("group", $group);
		$this->data("groupid", $group->id);		
		$this->data("user_list", $this->GroupsModel->get_users($group->id));
		$this->view("groupsDisplay", "groups/groupsingle_view");
		$this->style(Array('groups.css'));

		$this->display($group->name, "groups_view");
	}
	
	/**
	 * Post creation of a group.
	 */
	public function create() {
		if(!($this->AuthModel->checkUserLogin())) { redirect('/auth/register/'); }
		if($this->input->post('submitted')) { 
			$result = $this->_checkGroupSubmission($_POST);
			if(isset($result['errors'])) { $this->data("errors", $result["errors"]);}

			if($result["created"] == true) {
				$result = $this->_createGroup($_POST);	    
				redirect('group/'.url_title(strtolower($this->input->post("groupname")))); 
				} else { $title = "Create a Group &rarr; Please enter the correct information"; }
		}
		else { $title = "Create a Group"; }

		$this->style(Array('groups.css'));
		$this->view("groupsDisplay", "groups/groupscreate_view");
		$this->display($title, "groups_view");
	}  

	/**
	 * Enters the new members to the existing group.
	 */
	public function update() {
		$this->_enter_users($this->input->post('groupid'), null, $this->input->post('members'));
		$group = $this->GroupsModel->getGroupByID($this->input->post('groupid'));
		redirect('group/'.$group->slug);		
	}
	
	/**
	 * Creates the group and enters the members.
	 * @param $form array the submitted form data.
	 * @return array the group id of the created group.
	 */
	private function _createGroup($form) {
		$group_id = $this->GroupsModel->enterGroup($form['groupname'], $form['description']);	      
		$this->_enter_users($group_id, $form['admins'], $form['members']);

		return array("group_id" => $group_id);
	}
	 
	/**
	 * Checks if the form is set correctly.
	 * @param $form array the submitted form data.
	 * @return array the results status.
	 */	
	private function _checkGroupSubmission($form) {
		$errors = ""; $created = false;
	  
		if(isset($form['submitted'])){
			if(isset($form['groupname']) && isset($form['admins']) && isset($form['members'])) {
				$check = $this->_checkGroupName($form['groupname']);
			}
			if("" == $check) { $group_name = $form['groupname']; $created = true;}
			else { $errors = $check; $group_name =""; }	    	    
		}
	  
		return array("errors" => $errors,
		       	     "created"=> $created,
		       	     "group_name" => $group_name);
	}
	
	/**
	 * Checks if the group name is available
	 * @param $groupname string the groupname to check.
	 */
	private function _checkGroupName($groupname){
		$errors = "";
		if(!($this->GroupsModel->checkGroupName($groupname))){
			$errors .= "That groupname is unavailable, choose a different name.";
		}
		return $errors;
	}

	/**
	 * Takes the array of admins/member names and stores them in the members table with the correct id
	 * @param $group_id integer the group id to insert in members table
	 * @param $members admins a comma separated listed of admins
	 * @param $members string a comma separated listed of members (who are not admins)
	 */
	private function _enter_users($group_id, $admins, $members){	  
		$users = explode(",", str_replace(" ","", $admins));
		foreach ($users as $k => $v){
			$admin = $v;
			$this->GroupsModel->createAdmin($admin, $group_id);
		}
		$member_users = explode(",", str_replace(" ","", $members));
		foreach ($member_users as $k => $member){
			$this->GroupsModel->createMember($member, $group_id);
		}
	}
} // End Groups