<?php
/**
 * Controller for displaying/editing/adding objects.
 * 
 * @version 0.8.0
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage controllers
 * @uses AuthModel CommentsModel GroupsModel ObjectsModel TransportModel UsersModel googlemaps time locale text
 */

class Objects extends SM_Controller {
  private $locationCache = array('foo' => 'bar');
  private $transportCache = array();
  
  public function Objects(){
    parent::SM_Controller();
    $this->load->model(Array('ObjectsModel','TransportModel','AuthModel','UsersModel','CommentsModel', 'GroupsModel'));			
    $this->load->helper(Array('googlemaps', 'time', 'locale', 'text'));		
  }
  
	/**
	 * Default controller, loads a listing of objects
	 * @param $sort string how to sort the list (name, emissions, data, etc.)	
	 * @uses string
	 */
	public function index($sort="datedesc"){
		$this->load->helper(Array('string'));
		$this->load->library('pagination');
		
		$objectsort = $this->_getObjectSort($sort);
		
		$pagerange = $this->uri->segment(4);  
		$pageper = 10; 
		$pagetotal = $this->ObjectsModel->getObjectCount();
		$this->paginate($pageper,site_url() . '/objects/sort/'.$sort."/",$pagetotal,4);
					
		if($pagerange <= 1) { $this->data('infomessage', true); }
		
		$this->data('objectsArray', $this->ObjectsModel->getObjects($pageper, $pagerange, $objectsort));

		$this->data("sort", $sort);			

		$this->script(Array());
		$this->style(Array('objectlist.css'));			
		$this->view("objectsDisplay", "objects/objectslist_view");
		$this->contextual(Array('quickticket'));
		
		$this->display("Sourcemaps", "objects_view");
	}

	/**
	 * Informational page, describes different kinds of objects
	 */
	public function about() {
		$this->style(Array('objects.css'));			
		$this->view("objectsDisplay", "objects/objectabout_view");
		$this->display("Different Kinds of Sourcemaps", "objects_view");
	}
	
	/**
	 * Get Json from linked objects.
	 */
	public function getJSONByLinkID() {
		$linkid = $this->input->post('linkid');
		$object = $this->ObjectsModel->getObjectByLinkID($linkid);
		$this->output->set_output(json_encode($object));
	}

	/**
	 * Loads a single object representation, complete visualization with OpenLayers and map display.
	 * @param integer $id The oid number of the object.
	 * @param string $flag Used to trigger special actions, such as new object creation.
	 * @uses PowerModel
	 */
	public function single($slug=null, $type="object") {
		$this->load->model(Array('PowerModel'));
		if($slug== null) { show_404(); }
		$object = $this->ObjectsModel->getObject($slug); $id = $object->oid;
		if($object == null) {show_404(); }
				
		if($type == "view") { $type = $object->type;}
		$this->data("type", $type);
			
		// Get the creator and check editability
		$creatorid = $object->creator;
		$this->data("creator", $this->UsersModel->getUserById($creatorid));		
		$this->data("loggedin", $this->AuthModel->checkUserLogin());
              		
		// Editable returns permission type, group name that the current user belongs to and checks the permission
		if($this->AuthModel->checkUserLogin()) {
			$current_user = $this->AuthModel->getCurrentUserID();
			if($creatorid == $current_user || $this->AuthModel->getCurrentUserRole() == "admin") { $show = true;}
			else {$show = false;}
		} else { $show = false;}
		$result = $this->AuthModel->getCurrentPermissions($id);		
		$permission_object = $this->AuthModel->getObjectPermissions($id);
		
		/*Get the process already stored*/
		
		$processes = $this->ObjectsModel->getProcessesByOid($id);

		/**Get the endoflife already stored */
		$endoflife = $this->ObjectsModel->getEndoflifeByOid($id);
		
		$this->data("endoflife", $endoflife);
		$this->data("processes", $processes);
		$this->data("editable", $result["check"]);
		$this->data("permission_type", $result["permission_type"]);
		$this->data("groups", $result["groups"]);
		$this->data("permission_objects", $permission_object);
				

		$this->data("show", $show);
		$this->data("comments", $this->CommentsModel->getComments("object", $id));
		$this->data("favorited", $this->CommentsModel->checkFavorite($id));
		
		$this->data("object", $object);
		$this->data("transports", $this->TransportModel->getSimpleTransportList());
		$this->data("powersources", $this->PowerModel->getPowerList());
		
		$this->view("mapDisplay", "map/objectmap_view");
		$this->view("receiptDisplay", "visualizations/receipt");
		$this->view("partsDisplay", "visualizations/partslist");
		$this->view("pieDisplay", "visualizations/charting");			
		$this->view("commentsDisplay", "visualizations/comments");			
		$this->view("actionsDisplay", "visualizations/actions");
		
		$this->style(Array('objects.css', 'visualizations/main.css', 'visualizations/partslist.css', 'visualizations/charts.css', 'visualizations/receipt.css', 'visualizations/comments.css', 'visualizations/actions.css', 'visualizations/catalogue.css'));
	
		if($this->config->item('tileprovider') == "cloudmade") {
			$this->script(Array(getGoogleJsapi().getGoogleMapsKey(base_url()), 
								base_url()."assets/scripts/OpenLayers-2.8/OpenLayers.js", 
								base_url()."assets/scripts/OpenLayers-2.8/cloudmade.js"), true);
		} else { 
			$this->script(Array(getGoogleJsapi().getGoogleMapsKey(base_url()), 
								base_url()."assets/scripts/OpenLayers-2.8/OpenLayers.js"), true
								);		
		}				

		$this->script(Array('jquery/jquery.color.js', 'jquery/thickbox.js', 'jquery/jquery.qtip.js', 'jquery/jquery.enumerable.js', 'jquery/jquery.tufte-graph.js', 'jquery/raphael.js', 'util/json2.js', 'templates/'.$type.'.js', 'object.js', 'editor.js', 'comments.js', 'widgets.js', 'openlayers_map.js', 'catalogue.js'));
		$this->scriptvar(Array($this->ObjectsModel->generateJSONObject($object), 
							 "var permission_type = '".$result['permission_type']."';", 
							 "Sourcemap.tileprovider = '".$this->config->item('tileprovider')."';",
							 "Sourcemap.geocodeprovider = '".$this->config->item('geocodeprovider')."';"));
						
		if($object->emissions > 0) { $carbondescription = ' and has a carbon footprint of '.$object->emissions.' kg CO2e';}	
		else { $carbondescription = '';}
		$this->data('pagedescription', 'Sourcemap of '.$object->name.', created by '.$this->data['creator']->name.'. '.
					 				   'It comes from '.$object->origin.$carbondescription.'.');					
		$this->display($object->name, "objects/objectsingle_view");		
	}		
	
	/**
	 * Creates a skeleton object for the user to fill in.
	 * @param $type string the object template type.
	 * @param $error string any previously generated errors.
	 */
	public function create($type="object", $error='') {		
		if(!($this->AuthModel->checkUserLogin())) { redirect('/auth/register/'); }		
        
		$this->data("error", $error); 
		$this->data("type", $type);
		$creator = $this->UsersModel->getUserByName($this->AuthModel->getCurrentUser());
		$this->data("loggedin", $this->AuthModel->checkUserLogin());					
		
		$this->view("manualDisplay", "objects/objectcreate_manual_view");
		$this->view("gBaseDisplay", "objects/objectcreate_gbase_view");
		$this->view("excelDisplay", "objects/objectcreate_excel_view");
		
		$this->style(Array('create.css'));			
		$this->script(Array('jquery/jquery.qtip.js', 'util/json2.js', 'templates/'.$type.'.js', 'object.js', 'create.js', 'import.js','upclookup.js'));
		$this->script(Array("http://www.google.com/jsapi?autoload=%7B%22modules%22%3A%5B%7B%22name%22%3A%22maps%22%2C%22version%22%3A%222%22%7D%2C%7B%22name%22%3A%22earth%22%2C%22version%22%3A%221%22%7D%5D%7D&key=".getGoogleMapsKey(base_url())), true);

		$this->scriptvar(Array($this->ObjectsModel->generateJSONBlank($type, $creator->id)));				
		$this->display("Create a Sourcemap", "objects/objectcreate_view");
	}
	
	/**
	 * Ajax handler, gets google base results for upc codes.
	 */
	public function querygbase() {
		$this->load->library('GBase');	
		$upc = $this->input->get_post('upc');
		$results = $this->input->get_post('numresults');
		$feed = $this->input->get_post('feed');	
		$itemData = $this->gbase->queryUPC($upc, $results, $feed);
	
		if($upc != "") { $this->_displayGBasePreview($itemData, count($this->gbase->getStructuredData()));} 
		else { $this->_displayGBasePreview($itemData, 0);}
	}
	
	/**
	 * Loaded over ajax, triggers save of current edit object (passed by json)
	 */
	function save() {
	  $updatedObject = json_decode($this->input->post('data'));
	  $slug = $this->ObjectsModel->saveObject($updatedObject);
	  /* Clear caches for object. */
	  if(isset($updatedObject->oid)) {
	    if(substr($updatedObject->type,0,5) == "edit:") { $type = substr($updatedObject->type,5);} 
	    else { $type = $updatedObject->type;}
	    $this->clearCache("/objects/".$type."-".$updatedObject->oid);	
	    $this->clearCache("/objects/edit:".$type."-".$updatedObject->oid);					
	  }		
	  $this->output->set_output($slug);		
	}
	
	/**
	 * Called "secure_save" because it only allows for the editing of certain values
	 * but doesn't allow changing permissions, etc.
	 */
	function secure_save() {
		$data = $this->input->post('data');
		if (!isset($data) or $data == ''){
			$data = $this->input->post('sourcemap');
		}
		$this->log->write_log('info', 'saving object with data: '.$data);
		$updated = json_decode($data);
		$this->log->write_log('info', 'object oid is '.$updated->oid);
		$oids = array(); $oids[] = $updated->oid;
		$currentArr = $this->ObjectsModel->getObjectsByOids($updated->oid);
		$current = $currentArr[$updated->oid];
		$this->log->write_log('info', 'current object oid is '.$current->oid);
		$current->name = $updated->name == '' ? $current->name : $updated->name;
		$current->description = $updated->description == '' ? $current->description : $updated->description;
		$current->origin = $updated->origin == '' ? $current->origin : $updated->origin;
		//look up lat/lon based on location text
		$this->_populateLatLon($current);
		
		$partsById = array();
		foreach($updated->parts as $updPart){
			$this->log->write_log('info', 'hashing part'.$updPart->imid);
			$partsById[$updPart->imid] = $updPart;
		}
		foreach($current->parts as $curPart){
			$updPart = $partsById[$curPart->imid];
			if (isset($updPart)){
				$this->log->write_log('info', "updated part is $updPart->name for imid $curPart->imid"); 
				$curPart->name = $updPart->name == '' ? $curPart->name : $updPart->name;
				$curPart->origin = $updPart->origin == '' ? $curPart->origin : $updPart->origin;
				$curPart->weight = $updPart->weight == '' ? $curPart->weight : $updPart->weight;
				$curPart->rawemissions = $updPart->rawemissions == '' ? $curPart->rawemissions : $updPart->rawemissions;
				$curPart->emissions = $updPart->emissions == '' ? $curPart->emissions : $updPart->emissions;
				$curPart->shipping = $updPart->shipping == '' ? $curPart->shipping : $updPart->shipping;
				$curPart->via = $updPart->via == '' ? $curPart->via : $updPart->via;
				$curPart->description = $updPart->description == '' ? $curPart->description : $updPart->description;
				$this->_populateLatLon($curPart);
				$curPart->flag = 'changed';
			} 
			else { 
				$this->log->write_log('notice', "could not find part for imid $curPart->imid and object $updated->oid .  This should be impossible"); 
			}
		}
		
		$this->ObjectsModel->saveObject($current);
		$this->log->write_log('info', 'done saving object oid '.$updated->oid);

		//render an html fragment to allow for easily replacing the old version, AJAX-style
		$this->fullview = false; //suppresses output of headers, etc
		$this->data("object", $current);
		$this->display("Confirm Partial Import", "objects/objectsconfirmimport_readonly_partial");	
	}

	/**
	 * Ajax handler, checks permission type and sets the objects permission.
	 */
	public function setpermissions() {
		$data = json_decode($this->input->post("data"));
		$permission_type = utf8_decode($data->permission_type);
		$oid = utf8_decode($data->oid);

		if($permission_type == "useredit") {
			$user = $this->AuthModel->getCurrentUser();
			$user_id = $this->AuthModel->getCurrentUserID();
			$this->ObjectsModel->updatePermissions($permission_type, $oid, $user_id);
		} else if($permission_type == "groupedit") {
			$groupname = $data->group_name;
			$slugname = $this->GroupsModel->getSlugByName($groupname);
			$group_id =$this->GroupsModel->get_group_id($slugname);
			$this->ObjectsModel->updatePermissions($permission_type, $oid, $group_id);
		} else if($permission_type == "everyoneedit") {
			$this->ObjectsModel->updatePermissions($permission_type, $oid);
			// The visibility is automatically set to public if it is everyoneedit
			$this->ObjectsModel->updateObjectVisibility("public", $oid);
		}
	}



	/**
	 * Ajax handler, gets the list of processes
	 */
	
	public function getProcesses() {

	  $data = json_decode($this->input->post("data"));
	  $slug = utf8_decode($data->parts_slug);
	  $process_array = $this->ObjectsModel->getProcessData($slug);
	  $this->output->set_output(json_encode($process_array));

	}

	public function getEndoflife() {
	  $data = json_decode($this->input->post("data"));
	  $slug_endoflife_parts = utf8_decode($data->parts_endoflife_slug);
	  $endoflife_array = $this->ObjectsModel->getEndoflifeData($slug_endoflife_parts);
	  $this->output->set_output(json_encode($endoflife_array));
 
	}



	/**
	 * Ajax handler, sets objects visibility.
	 */
	public function setvisibility() {
		$data = json_decode($this->input->post("data"));
		$oid = utf8_decode($data->oid);
		$visibility = utf8_decode($data->visibility); 

		if ($visibility == "public") { $this->ObjectsModel->updateObjectVisibility($visibility, $oid);}
		else if($visibility == "private") {
			$res = $this->AuthModel->getCurrentPermissions($oid);
			if(($res["permission_type"]) == "everyoneedit") {
				$visibility = "public";
				$this->ObjectsModel->updateObjectVisibility($visibility, $oid);
			} else { $this->ObjectsModel->updateObjectVisibility($visibility, $oid);}
		}
	}
	
	/**
	 *
	 */
	public function confirm_import(){
		if(!($this->AuthModel->checkUserLogin())) {
			redirect('/auth/register/');  
		}
		$oidList = $this->input->post('oid');
		$this->log->write_log('info', "looking for object in this oid list: ");
		foreach($oidList as $oid){
			$this->log->write_log('info', $oid);
		}
		$updatedObjects = $this->ObjectsModel->getObjectsByOids($oidList);
		$success = true;
		foreach ($updatedObjects as $updatedObject){
			if ($this->ObjectsModel->validate($updatedObject)){
				//save to database
				$this->ObjectsModel->saveObject($updatedObject, false);
			} else {
				//give user useful error message
				$error .= $updatedObject->explain_validation_errors()."<br/>";
			}
		}
		if ($success){
			$this->index("datedesc");
		} else {
			$this->_initConfirmImportView($updatedObjects, $error);
			$this->display("Confirm Import", "objects_view");
		}
	}
	

	/* --- Import Controllers -------------------------------------------------------------------------- */
	private function _initImportView($error=''){
		$this->data("error", $error);
		$creator = $this->UsersModel->getUserByName($this->AuthModel->getCurrentUser());						
		$this->data("loggedin", $this->AuthModel->checkUserLogin());					
	
		$this->view("objectsDisplay", "objects/objectimport_view");
		$this->style(Array('import.css'));			
		$this->script(Array('jquery/jquery.qtip.js', 'util/json2.js', 
								'templates/object.js', 'import.js'));
	}
	
	private function _initConfirmImportView($candidateObjects, $error=''){
		$this->data("error", $error);
		$creator = $this->UsersModel->getUserByName($this->AuthModel->getCurrentUser());						
		$this->data("loggedin", $this->AuthModel->checkUserLogin());
		$this->data('objectsArray', $candidateObjects);			
		$this->view("objectsDisplay", "objects/objectsconfirmimport_view");
		$this->style(Array('import.css','objectlist.css'));			
		$this->script(Array('jquery/jquery-1.3.min.js','jquery/jquery.qtip.js'));
		$this->script(Array('util/json2.js'));
		$this->script(Array('templates/object.js', 'object.js'));
		$this->script(Array('import.js'));		
	}

    public function excelImport($type="object"){ 
		if(!($this->AuthModel->checkUserLogin())) {
			redirect('/auth/register/');  
		}
		$this->load->library('excel_reader2');
		$this->load->library('SMPartsParser');
	
		//initialize uploader
		//$config['uploadpath'] = 'uploads/';
		$config['allowed_types'] = 'xls';
		$config['max_size'] = '10000';
		$this->load->library('upload', $config);
		$this->upload->upload_path = 'assets/uploads/';
		$error = '';
		$success = false;		
		if (! $this->upload->do_upload()){
			$error .= $this->upload->display_errors()."<br/>";
		} else {
			//save uploaded spreadsheet to a temp file
			$file_data = $this->upload->data();
			$path = $file_data['full_path'];
			$this->log->write_log('info', "saved temp spreadsheet to $path");
			//decode spreadsheet into json object
			$parser = new SpreadsheetXLSParser();
			$jsonValue = $parser->to_canonical($path, $type);
			$this->log->write_log('info', 'got this json:'.$jsonValue);
			$updatedObjects = json_decode($jsonValue);
			$this->log->write_log('info', "decoded this many objects: ".sizeof($updatedObjects));
			if (sizeof($updatedObjects) < 1){ 
				$error .= "We're sorry, it appears this spreadsheet has no objects, or it is in the wrong format.  Please check it and try again<br/>"; 
				$error .= $parser->getErrors(); 
				$success = false; 
			} else { 
				$this->log->write_log('info'," with this many parts in the first object: ".sizeof($updatedObjects[0]->parts)); 
				$oids = array(); 
				$savedObjects = array();         
				foreach ($updatedObjects as $updatedObject){ 
					//validate object and enrich as necessary 
					$updatedObject = $this->_enrich($updatedObject, true); 
					//save to database 
					$this->ObjectsModel->saveObject($updatedObject); 
					$oids[] = $updatedObject->oid; 
				} 

				//I'm refetching them here because I need the imid for each part and those are auto generated by the database 
				$savedObjects = $this->ObjectsModel->getObjectsByOids($oids); 

				$this->_checkTransports($savedObjects); 

				$transports = $this->TransportModel->getSimpleTransportList(); 
				$this->data('transports', $transports); 

				//TODO: clean up temporary files 
				$success = true;
			}		
		}
	
		if ($success){
			$this->_initConfirmImportView($savedObjects);
			$this->display("Confirm Import", "objects_view");
		} else {
	        $this->create('object',$error); 
		}
	}

	private function _checkTransports($objects){
		//look up lat/lon for each part also based on location text
		foreach ($objects as $skeleton){
			foreach ($skeleton->parts as $part){
				//check that transport methods actually match something we know in the database
				if (!array_key_exists($part->via,$this->transportCache)){
					$this->log->write_log('info', "Validating transport against database ".$part->via);
				
					$transport = $this->TransportModel->getTransportByName($part->via);
					if (!isset($transport)){
						$part->invalid_transport = true;	
					} else {
						$part->invalid_transport = false;
					}
					$this->transportCache[$part->via] = $part->invalid_transport;
			} else {
					$part->invalid_transport = $this->transportCache[$part->via];
				} 
			}
		}		
	}

	private function _enrich($skeleton, $added=false){
		$creator = $this->UsersModel->getUserByName($this->AuthModel->getCurrentUser());
		$skeleton->creator_id = $creator->id;						
		$skeleton->creator = $creator->id;	
		// Todo are these the same ?
	
		if ($added)	$skeleton->flag = 'added';
		if (!isset($skeleton->upc))	$skeleton->upc = '';
		if (!isset($skeleton->imagelink))	$skeleton->imagelink = '';
		if ($added){
			foreach ($skeleton->parts as $part){
				$part->flag = 'add';
			}
		}
	
	
		//calculate any energies needing calculating
		$skeleton = $this->_recalculateObjectValues($skeleton);
	
		//look up lat/lon based on location text
		$this->_populateLatLon($skeleton);
	
		//look up lat/lon for each part also based on location text
		foreach ($skeleton->parts as $part){
			$this->_populateLatLon($part);
		}
	
		return $skeleton;	
	}

	private function _populateLatLon($lookmeup){
		try {
			$coordinates = $this->_lookupLatlong($lookmeup->origin);
			$lookmeup->latlon = $coordinates[1]."|".$coordinates[0];
			$lookmeup->originalorigin = $lookmeup->origin;
			$lookmeup->origin = $coordinates[2];
			//TODO: also populate the origin field with the full place name
		} catch (Exception $e){
			$this->log->write_log('notice', "caught an exception trying to do Google lookup for object: $e");
		}
	}

	private function _recalculateObjectValues($skeleton){
		$emissionsTotal = 0;
		$shippingDistanceTotal = 0;
		$weighttotal = 0;
		$transportEmissionsTotal = 0;
		$transportKGKM = 0;
		$embodiedEmissionsTotal = 0;
	
		foreach ($skeleton->parts as $part){
			$embodiedEmissionsTotal += $part->emissions;
			$transportEmissionsTotal += $part->shipping * ($part->weight/1000) * $part->transportemissions;
			$weighttotal += $part->weight;
			$shippingDistanceTotal += $part->shipping;
			$transportKGKM += $part->weight * $part->shipping;	
		}
	
		if ($skeleton->showembodied == "on") { $emissionsTotal += $embodiedEmissionsTotal;}
		if ($skeleton->showtransport == "on"){ $emissionsTotal += $transportEmissionsTotal;}
		if ($skeleton->showuse == "on" and isset($sekelton->usageemissions)){ $emissionsTotal += $skeleton->usageemissions;}
		$skeleton->emissions = $emissionsTotal;
		$skeleton->embodiedEmissions = $embodiedEmissionsTotal;
		$skeleton->transportEmissions = $transportEmissionsTotal;
		$skeleton->shipping = $shippingDistanceTotal;
		$skeleton->weight = $weighttotal;
	
		return $skeleton;	
	}

	/*
	 * Attempts to look up an object's location using Google
	 */
	private function _lookupLatlong($address){
		$address = urlencode($address);
		if (!array_key_exists($address,$this->locationCache)){
			$this->log->write_log('info', "searching Google for location: $address");
			// Basic geocoder use google api key as $key
			$key = "";
	
			$url = "http://maps.google.com/maps/geo?q=".$address."&output=xml&key=".$key;
			$ch = curl_init();
	
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_HEADER,0); //Change this to a 1 to return headers
			curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER["HTTP_USER_AGENT"]);
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	
			$data = curl_exec($ch);
			//echo $data;
			curl_close($ch);
			// after you run this function you can walk the $xml
			$this->log->write_log('info', "got from google:". $data);
			$xml = new SimpleXMLElement($data);
			if (isset($xml->Response->Placemark) and isset($xml->Response->Placemark->Point)){
				$coordinates = $xml->Response->Placemark->Point->coordinates[0];
				$coordinates = explode(",", $coordinates);
				$fullAddress = $xml->Response->Placemark->address[0];
				$coordinates[2] = $fullAddress;
			} else {
				$coordinates = array(0,0,'');
			}
			$this->locationCache[$address] = $coordinates;
			$this->log->write_log('info', "location cache is now ". $this->locationCache[$address]);
		} else {
			$coordinates = $this->locationCache[$address];
		}
	
		return $coordinates;
	}
	
	/**
	 * Gives the sql sort for a simple language sort term.
	 * @param $sort string the simple language sort term.
	 * @return string the sql representation.
	 */
	private function _getObjectSort($sort) {
		if($sort == "date") { $objectsort = 'objects_complex.timemodified ASC'; }			
		if($sort == "name") { $objectsort = 'objects_complex.name ASC'; }
		if($sort == "weight") { $objectsort = 'objects_complex.weight ASC'; }			
		if($sort == "emissions") { $objectsort = 'objects_complex.emissions ASC'; }
		if($sort == "shipping") { $objectsort = 'objects_complex.shipping ASC'; }
					
		if($sort == "datedesc") { $objectsort = 'objects_complex.timemodified DESC'; }			
		if($sort == "namedesc") { $objectsort = 'objects_complex.name DESC'; }
		if($sort == "weightdesc") { $objectsort = 'objects_complex.weight DESC'; }			
		if($sort == "emissionsdesc") { $objectsort = 'objects_complex.emissions DESC'; }
		if($sort == "shippingdesc") { $objectsort = 'objects_complex.shipping DESC'; }
		
		return $objectsort;
	}
	

	/**
	 * Creates an html preview of google base data.
	 * @param $itemData string the result from google base.
	 * @param $resultsProcessed integer the number of results.
	 */
	private function _displayGBasePreview($itemData, $resultsProcessed) {
		$out = "";
		$out .= "<label>Preview:</label><br/>";
		if($resultsProcessed == 0) {
			$out .= "<div style='float:left'>No item data found for this UPC.</div>";
		} else {
			$out .= "<div style='float:left;'><img height=90 width=90 id='pimage' src=".$itemData['imgsrc']." style='float:left;margin:3px 5px 0 0;'/>";
			$out .= "<strong>Name</strong>: "."<a href='".$itemData['url']."'><span id='pname'>".$itemData['name']."</span></a><br/>";
			$out .= "<strong>UPC</strong>: ".$itemData['upc']."<br/>";
			$out .= "<strong>Est. Weight</strong>: ".$itemData['weight']."<br/>";
			$out .= "<strong>Est. Dimensions</strong>: ".$itemData['length'];
			$out .= " x ".$itemData['width']." x ".$itemData['height']."</div><br/>";
		}
		$this->output->set_output($out);
	}

	/**
	 * Creates a verbox html preview of google base data.
	 * @param $itemData string the result from google base.
	 * @param $resultsProcessed integer the number of results.
	 * @param $xml string the initial xml.
	 */
	private function _displayGBaseVerbose($itemData, $resultsProcessed, $xml) {
		$out = "";
		$out .= "<strong>Name</strong>: ".$itemData['name']."<br/>";
		$out .= "<strong>Description</strong>: ".$itemData['desc']."<br/>";
		$out .= "<strong>UPC</strong>: ".$itemData['upc']."<br/>";
		$out .= "<strong>Median Price</strong>: $".$itemData['medianprice']."<br/>";
		$out .= "<strong>Dimensions (LxWxH)</strong>: ".$itemData['length'];
		$out .= " x ".$itemData['width']." x ".$itemData['height']."<br/>";
		$out .= "<strong>Est. Weight (lbs.)</strong>: ".$itemData['weight']."<br/>";
		$out .= "<strong>Location</strong>: ".$itemData['loc']."<br/>";
		$out .= "<strong>Results processed</strong>: ".strval($resultsProcessed)."<br/>";
		$out .= "<a href='".$itemData['url']."'>Link to item</a><br/>";
		$out .= "<a href='".$itemData['imgsrc']."'>Link to image</a><br/>";
		$out .= "---<br/><a href ='";
		$out .= site_url()."portal'>";
		$out .= "Back to Google Base Portal</a><br/>---<br/>";
		$out .= "Raw Data (XML):<br/>";
		$out .= $xml;
		$this->output->set_output($out);
	}
}

?>
