<?php
/**
 * Controller for users.
 *
 * @version 0.8
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage controllers
 */
class Users extends SM_Controller{	
	function Users(){
		parent::SM_Controller();
		$this->load->model(Array('UsersModel', 'AuthModel'));
		$this->load->helper('time');				
	}
	
	/**
	 * Default controller, loads a listing of users
	 */
	function index(){		
		$this->load->helper('string');			
		$this->load->library('pagination');
		$this->data("current", "objects");
		
		$range = $this->uri->segment(2); 
		$pageper = 30; 
		$pagetotal = $this->UsersModel->getUsersCount();			
		$this->paginate($pageper,site_url()."users/",$pagetotal,2);
		if($range == null) { $range = 0;}
		$this->data("usersArray", $this->UsersModel->getUsers($pageper, $range));
		if($range <= 1) { $this->data('infomessage', true); }						
		
		$this->view("usersDisplay", "users/userslist_view");			
		$this->style(Array('users.css'));
		$this->contextual(Array());			

		$this->display("Members", "users_view");
	}
	
	/**
	 * Loads a single user representation.
	 * @param string $name A string representing the user name.
	 */
	function single($name=null) {
		$this->data("current", "objects");
				
		if($name == null ||  $this->AuthModel->getCurrentUser() == $name) { $this->dashboard(); }
		else {
			$this->load->helper(Array('string','text'));			
			$this->load->model('ObjectsModel');

			$user = $this->UsersModel->getUserByName($name);
			if($user == null) {show_404(); }
		
			$this->data('user', $user);
			$this->data('objectsArray', $this->ObjectsModel->getObjectsByUserID($user->id));
		
			$this->view("objectsDisplay", "objects/objectslist_view");						
			$this->view("usersDisplay", "users/userssingle_view");			
			$this->style(Array('users.css', 'objectlist.css'));
			$this->contextual(Array());			
			$this->display($user->name, "users_view");
		}
	}
	
	/**
	 * Loads the dashboard for the active user.
	 */
	function dashboard() {
		$this->data("current", "dashboard");
		
		$this->load->helper(Array('url', 'text', 'string'));
		$this->load->model(Array('ObjectsModel', 'GroupsModel'));
		
		if($this->AuthModel->getCurrentUserRole() == "admin") { $this->data('administrator', true); }
		else { $this->data('administrator', false); }
					
		if($this->AuthModel->checkUserLogin()) {	
			$user = $this->UsersModel->getUserByName($this->AuthModel->getCurrentUser());
			$this->data('apikey', md5("supplyisdemand" . $this->AuthModel->getCurrentUserEmail()));			
			$this->data('user', $user);
			$this->data('lastlogin', $this->AuthModel->getCurrentUserLastLogin());
			
			$created = $this->ObjectsModel->getObjectsByUserID($user->id);	
			$this->data('objectsArray', $created);					
			$this->view("objectsDisplay", "objects/objectslist_view");			
			
			$favorites = Array("objectsArray"=>$this->ObjectsModel->getFavoritedObjectsByUserID($user->id));			
			$this->view("favsDisplay", "objects/objectslist_view", $favorites);			
			
			$comments = Array("objectsArray"=>$this->ObjectsModel->getCommentedObjectsByUserName($user->name));
			$this->view("commentsDisplay", "objects/objectslist_view", $comments);			
			
			$groups = Array("groupsArray"=>$this->GroupsModel->getUsersGroups($user->id));
			$this->view("groupsDisplay", "groups/groupslist_view", $groups);

			$this->data("createdcount", count($created));			
			$this->data("favcount", count($favorites['objectsArray']));
			$this->data("commentcount", count($comments['objectsArray']));
			$this->data("groupcount", count($groups['groupsArray']));
			
			$this->style(Array('dashboard.css', 'objectlist.css', 'groups.css'));
			$this->script(Array('dashboard.js'));
			
			$this->contextual(Array());			

			$this->display($this->AuthModel->getCurrentUser(), "dashboard_view");	
		}
		else { redirect('/auth/register/'); }
	}		
	
	/**
	 * Updates the user's profile.
	 */
	function updatedashboard() {
		$profile = json_decode($this->input->post("data"));
		$this->UsersModel->updateUserProfile($profile);
	}
}
