<?php
//require_once('excel_reader2.php');
/*
 * Created on Nov 18, 2009
 *
 * @author Tim Cull
 */
 class SMPartsParser {
	private $_has_started = false;
	private $_first_part = true;
	private $_part_count = 0;
	private $_first_object = true;
	protected $_parseErrors = "";
	
	function _startObject($author, $name, $description, $origin_name, $type){
		$rv = "";
		if (!$this->_first_object){
			$rv = ",";
		} 
		$this->_first_object = false;
	$rv .= '{"oid":"",' .
			'"type":"'.$type.'",' .
			'"flag":"",' .
			'"permissions":":",';
	$rv .= '"name":"'.$name.'",' .
			'"description":"'.$description.'",' .
			'"category":"",'.
			'"upc":"",'.
			'"image_link":"",';
	$rv .= '"origin":"'.$origin_name.'",' .
			'"latlon":"0|0",';
	$rv .= '"destination":"",' .
			'"weight":"",' .
			'"emissions":"",';
	$rv .= '"embodied":"",' .
			'"transport":"",' .
			'"shipping":"",';
	$rv .= '"via":"",' .
			'"creator":"'.$author.'",' .
			'"lifetime":"0",' .
			'"usagetype":"",' .
			'"usageenergy":"0",';
	$rv .= '"usageemissions":"0",' .
			'"timecreated":"",';
	$rv .= '"timemodified":"",' .
			'"visibility":"public",';
	$rv .= '"showfootprint":"on",' .
			'"showembodied":"on",' .
			'"showtransport":"on",';
	$rv .= '"showprocess":"on",' .
			'"showendoflife":"on",' .
			'"showlastleg":"on",';
	$rv .= '"showuse":"on",' .
			'"showstats":"on",' .
			'"parts":[';
		$this->_has_started = true;
		return $rv;
	}

	function _doPart($name, $description, $origin_name, $weight, $emissions ,
			$embodied_energy, $transport_energy, 
			$transport_name, $shipping_emissions){
			$rv = $this->_first_part ? "" : ",";//prefix a comma, except for the first one in the parts list.
			$rv .= '{"name":"'.$name.'",' .
			'"linkid":"0",' .
			'"description":"'.$description.'",' .
			'"origin":"'.$origin_name.'",' .
			'"latlon":"0|0",' .
			'"via":"'.$transport_name.'",' .
			'"weight":"'.$weight.'",' .
			'"rawemissions":"'.$embodied_energy.'",' .
			'"emissions":"'.$emissions.'",' .
			'"shipping":"'.$shipping_emissions.'",' .
			'"transportemissions":"'.$transport_energy.'",' .
			'"link":"false"}';
			$this->_first_part = false;
			$this->_part_count++; 	
			return $rv;
	}

	function _finishObject(){
		$rv = "";
		if ($this->_has_started){
			$rv = '],"partcount":'.$this->_part_count.'}'; 	
		}
		$this->_has_started = false;
		$this->_first_part = true;
		$this->_part_count = 0;
		return $rv;
	} 
 	
 	function _startHeader(){
 		return "[";
 	}
 	
 	function _finishHeader(){
 		return "]";
 	}
 	
 	function getErrors() {
 		return $this->_parseErrors;
 	}
 }
 
 class SpreadsheetXLSParser extends SMPartsParser {
 	/**
 	 * Takes in a path to a binary Microsoft Excel File and returns back a
 	 * string in canonical format 
 	 * 
 	 */
 	function to_canonical($path, $type){
		$excel = new Spreadsheet_Excel_Reader($path,false);
 		$rv = $this->_startHeader();
		$x=1;
		$numcols = $excel->sheets[0]['numCols'];
		$numrows = $excel->sheets[0]['numRows'];
	    while($x<= $numrows) {
	    	$row = isset($excel->sheets[0]['cells'][$x]) ? $excel->sheets[0]['cells'][$x] : array();
			if ($this->_isObject($row)){
				$rv .= $this->_finishObject();
				$rv .= $this->_startObject($row, $type);
			} else if ($this->_isPart($row)){
				$rv .= $this->_doPart($row);
			}
			$x++;
		}
		$rv .= $this->_finishObject();
		$rv .= $this->_finishHeader(); 
		return $rv;	
 	}

	function _isObject($row){
		return isset($row[1]) ? $row[1] === 'object' : false;
	}
	
	function _isPart($row){
		return isset($row[1]) ? $row[1] === 'part' : false;
	}

	/* expects the following field order in spreadsheet:
	 * Type	
	 * Author	
	 * Sourcemap Name	
	 * Hub 
	 * Location	
	 * Description								
	 */
	function _startObject($row, $type){
		return parent::_startObject($this->esc($row, 2),$this->esc($row, 3),$this->esc($row, 5),
			$this->esc($row, 4), $type);
		
	}

	function esc($arg, $i){
		$rv =  isset($arg[$i]) ? $arg[$i] : '';
		$rv = preg_replace('/[^A-Za-z0-9 _\,\/\-\+\&\"\']/','',$rv);
		$rv = str_replace('"', '\"', $rv);
		return $rv;
	}
	
	/* expects following field order in spreadsheet:
	 * Type	
	 * Name	
	 * Description	
	 * Category	
	 * Origin Name	
	 * Weight	
	 * Emissions	
	 * Embodied Energy	
	 * Transport Energy	
	 * Transport Name	
	 * Shipping Emissions
	 */
	function _doPart($row){
		return parent::_doPart($this->esc($row, 2),$this->esc($row, 3),$this->esc($row, 4),
			$this->esc($row, 5),$this->esc($row, 6),$this->esc($row, 7),
			$this->esc($row, 8),$this->esc($row, 9),$this->esc($row, 10));
	}
 }
 
?>
