<?php

/**
 * Model for handling authentication.
 * 
 * @author Matthew Hockenberry
 * @package sourcemap
 * @subpackage models
 */


class AuthModel extends Model{

  /**
   * @ignore
   */	function AuthModel(){
		parent::Model();	
		$this->load->library('SimpleLoginSecure');
	}

    /*
     *  Returns where the user is logged_in or not
     */
    function checkUserLogin() {
      if($this->session->userdata('logged_in')) { return true; }
      else {return false;}		
    }
    
    /**
     * Gets the logged_in user name and checks with the session data. 
     * returns true if correct otherwise false.
     */
    function getCurrentUser() {
      if($this->checkUserLogin()) { return $this->session->userdata('name'); } else { return false; }
    }
    
    /**
     * Returns true if the current user id is same as the session user id
     */
    function getCurrentUserID() {
      if($this->checkUserLogin()) { return $this->session->userdata('id'); } else { return false; }
    }
    
    /**
     * Checks if the user data is role to set it to admin and display the dashboard.
     */
    function getCurrentUserRole() {
      if($this->checkUserLogin()) { return $this->session->userdata('role'); } else { return false; }
    }
    
    /**
     * Checks the last modified data.
     */
    function getCurrentUserLastLogin() {
      if($this->checkUserLogin()) { return $this->session->userdata('modified'); } else { return false; }
    }
    
    /**
     * Returns the current user email if it exists.
     */
    function getCurrentUserEmail() {
      if($this->checkUserLogin()) { return $this->session->userdata('email'); } else { return false; }
    }
    
    /**
     * When clicked on logout, it ends the current session of the user.
     */
    function logoutCurrentUser() {
      $this->simpleloginsecure->logout();		
    }
    
    /**
     * Checks the authenticity of the user name and password and
     * creates session for the user
     */
    function loginCurrentUser($username, $password) {
      if($this->simpleloginsecure->login($username, $password)) {
	return "true";
      } 
      else { return "false"; }
    }

    function changeCurrentUserPassword($old, $new) {
      if($this->simpleloginsecure->changepass($old, $new)) {
	return "true";
      } 
      else { return "false"; }
    }

    function setCurrentUserPassword($new, $email) {
      $this->simpleloginsecure->setpass($new, $email); 
    }
    /**
     * Creates an user with the entered data,
     * this data is entered into the database.
     */
    function createUser($user, $pass, $email, $description, $contact) {
		if($contact == 'TRUE') { $this->addToMailchimp($email, $user);}
      $this->simpleloginsecure->create($user, $email, $pass, $description, $contact);		
      $this->emailRegistration($user, $pass, $email, $description, $contact);
    }
    
    private function addToMailchimp($email, $username) {
	  	$this->load->library(Array('MCAPI')); 
	
 		$listId = '2a7825308e';

		$merge_vars = array('FNAME'=>$username);

		$retval = $this->mcapi->listSubscribe($listId, $email, $merge_vars, $email_type='html', $double_optin=false, $update_existing=true, $replace_interests=true, $send_welcome=false);

		if ($this->mcapi->errorCode){
	
		} 
	}
	
    public function resetForgottenPassword($email, $user) {
      $newpass = $this->createRandomPassword();
      $this->setCurrentUserPassword($newpass, $email);
      
      $this->load->library('email');

      $this->email->from('sourcemap@media.mit.edu', 'The Sourcemap Team');
      $this->email->to($email); 

      $this->email->subject('Sourcemap Password Reset');
      $this->email->message('Hi '.$user.'. We have generated a new password for you on Sourcemap.
Your new password is '.$newpass.'. 
If you have any questions or problems be sure to contact us at sourcemap@media.mit.edu');	

      if ( ! $this->email->send())
	{
	  echo "Error with your email address!";
	}
	}
	
	private function createRandomPassword() { 
	    $chars = "supplychainmapearthglobe!!02345678"; 
	    srand((double)microtime()*1000000); 
	    $i = 0; 
	    $pass = '' ; 

	    while ($i <= 7) { 
	        $num = rand() % 33; 
	        $tmp = substr($chars, $num, 1); 
	        $pass = $pass . $tmp; 
	        $i++; 
	    } 

	    return $pass; 
	}
	
    /**
     * Returns the current permissions of the object
     * @param int $oid
     */
    
    public function getCurrentPermissions($oid) { 
      $this->db->select('*');
      $this->db->from('permissions');
      $this->db->where('oid_number', $oid);
      $query = $this->db->get();
      $permissions = array_pop($query->result());
      
      if(!($this->checkUserLogin())) 
	{ 
	  return array("check" => false, "permission_type" => "", "groups" => null); 
	}
      if(!($permissions)) 
	{
	  if($this->getCurrentUserRole() == "admin") {
	    return array("check" => true, "permission_type" => "useredit", "groups" => $this->getCurrentUserGroups());
	  } else {
	    return array("check" => false, "permission_type" => "useredit", "groups" => $this->getCurrentUserGroups());
	  }
	}
      
      if($permissions->permission_type == "useredit") {return $this->getUserPermissions($permissions);}
      if($permissions->permission_type == "groupedit") {return $this->getGroupPermissions($permissions);}
      if($permissions->permission_type == "everyoneedit") {return $this->getEveryonePermissions($permissions);}
    }
    

    /**
     * Gets the current visibility of the object 
     * @param int $oid
     */
    public function getObjectPermissions($oid) {

      $this->db->select('visibility');
      $this->db->from('objects_complex');
      $this->db->where('oid', $oid);
      $query = $this->db->get();
      $permission_object = array_pop($query->result())->visibility;

      return $permission_object;

    }
    
    /**
     * Checks if the current user permissions is useredit 
     * @param string $permissions
     * @return check, permission_type, groups
     */
    private function getUserPermissions($permissions) {
      $id = $permissions->id_number;
      $idcheck = $this->getCurrentUserID();
      if($id == $idcheck || $this->getCurrentUserRole() == "admin") { $check = true; } 
      else { $check = false; }
      
      return array("check" => $check,
		   "permission_type" => "useredit",
		   "groups" => $this->getCurrentUserGroups());
    }
    
    
    /**
     * Checks whether the current user has group permissions
     * @param string $permissions
     * @return $check=true only if the user is admin in the group or
     * the current user is the memeber of the group,
     * permission_type, groups that user is associated with
     */
    private function getGroupPermissions($permissions) {
      $currentUserName = $this->getCurrentUser();
      $this->db->select('username');
      $this->db->from('members');
      $this->db->where('group_id', $permissions->id_number);
      $query = $this->db->get();
      $members = $query->result();
      $check = false;
      if($this->getCurrentUserRole() == "admin") {
	$check = true;
      } else {
	foreach($members as $user){
	  if($user->username == $currentUserName) { $check = true;}
	}
      }
      
      return array("check" => $check,
		   "permission_type" => "groupedit",
		   "groups" => $this->getCurrentUserGroups());			
    }
    
    
    /**
     * Returns true and sets the permission_type as everyoneedit
     */
    private function getEveryonePermissions() {
      return array("check" => true,
		   "permission_type" => "everyoneedit",
		   "groups" => $this->getCurrentUserGroups());
    }
    
    /**
     * Returns all the name of groups that current user belongs to
     */
    private function getCurrentUserGroups() {
      $user = $this->getCurrentUser();
      $this->db->select('name');
      $this->db->from('groups');
      $this->db->join('members','members.group_id = groups.id');
      $this->db->where('members.username', $user);
      $query = $this->db->get();
      return $query->result(); 
    }
    
    /**
     * Sends an email confirming the registration of the user.
     * If the email address does not exist then it sends an error message.
     */
    function emailRegistration($user, $pass, $email, $description, $contact) {
      $this->load->library('email');
      
      $this->email->from('sourcemap@media.mit.edu', 'The Sourcemap Team');
      $this->email->to($email); 
      
      $this->email->subject('Welcome to Sourcemap.');
      $this->email->message('Hi '. $user . ' thanks for registering for Sourcemap.
You can always visit www.sourcemap.org to create and edit objects. 
If you have any questions or problems be sure to contact us at sourcemap@media.mit.edu');	
      
      if ( ! $this->email->send())
	{
	  echo "Error with your email address!";
	}
    }
    
    /**
     * 
     * @todo Should probably do something.
     */
    function checkAPIKey($key) {
      return true;
    }
  }
?>