<?php

/**
 * Representation of transport methods as model
 * 
 * @package sourcemap
 * @subpackage models
 */

class TransportModel extends Model{
	
	/**
	 * @ignore
	 */
	function TransportModel(){
		parent::Model();	
	}	
	
	/**
	 * Returns the result from transport table
	 */
	function getSimpleTransportList(){
			$this->db->select('name, description, co2_eq');		
			$this->db->from('transport');

			$query = $this->db->get();
			return $query->result();
	}
	
	/**
	 * Returns the result from the transport table
	 * @param string $name
	 */
	function getEmissionsFromTransport($name) {
		$this->db->select('co2_eq');		
		$this->db->from('transport');
		$this->db->where('name', $name);
		
		$query = $this->db->get();
		$raw = array_pop($query->result());
		if(!isset($raw->co2_eq)) { $raw->co2_eq = 0;}
		return $raw->co2_eq;
	}

	function getTransportByName($name){
		$this->db->select('*');		
		$this->db->from('transport');
		$this->db->where('name', $name);
		
		$query = $this->db->get();
		$raw = array_pop($query->result());
		return $raw;
	}

	/**
	 * Gets the list of all transport methods.
	 * @return Array Array of all transport methods.
	 * @param integer $limit Limit number of transport methods.
	 * @param integer $offset Offset to return transport methods.
	 * @param string $sort How transport methods should be sorted (name ASC).
	 */
	function getTransportMethods($limit = null, $offset = null, $sort = 'name ASC'){
		$this->db->select('*');		
		$this->db->from('transport');
		
		$this->db->order_by($sort);		
		
		$this->db->limit($limit, $offset);
		
		$query = $this->db->get();
		return  $query->result();
	}
	
	/**
	 * Returns the total size of the transport methods table.
	 * @return int Number of transport methods.
	 */
	function getTransportCount(){
		return $this->db->count_all_results('transport');
	}
}
?>