/**
 * Sourcemap.ObjectCreator: Handles creation and initialization of Sourcemap Objects.
 *
 * @version 0.8
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage js
 */

if ( typeof(Sourcemap) == 'undefined' ) {Sourcemap = {};}

Sourcemap.ObjectCreator = {
  savingopen: true,
  savingrequested: false,

  init: function() {	
	SMap = {};	
	SMap.geoCoder = new GClientGeocoder();
	SMap.setOrigin = function() { };
	SMap.refreshMap = function() { };
	
	$(".createbutton").click(function() { Sourcemap.ObjectCreator.save(); });
	
	$(".input-object-name").change(function() { 
		objectConfig.setName($(this).val()); $(this).addClass("dirty");
	});
	$(".input-object-description").change(function() { 
		objectConfig.setDescription($(this).val()); $(this).addClass("dirty");
	});
	$(".input-object-origin").change(function() { 
		objectConfig.setOrigin($(this).val()); $(this).addClass("dirty");
	});
	
	$(".input-upcobject-name").change(function() { 
		objectConfig.setName($(this).val()); $(this).addClass("dirty");
	});
	
	$(".input-upcobject-description").change(function() { 
		objectConfig.setDescription($(this).val()); $(this).addClass("dirty");
	});
	
	$(".input-upcobject-origin").change(function() { 
		objectConfig.setOrigin($(this).val()); $(this).addClass("dirty");
	});
	
	$(".input-upcobject-upc").change(function() { 
		objectConfig.setUPC($(this).val()); $(this).addClass("dirty");
	});
	
	$("body").bind("originGeoMessage", function(e, id, msg) { 
		Sourcemap.ObjectCreator.receiveGeocodeMessage($(".input-object-origin"), msg);
		Sourcemap.ObjectCreator.receiveGeocodeMessage($(".input-upcobject-origin"), msg);
	});	
	
	$("body").bind("originGeoError", function(e, id) {
		Sourcemap.ObjectCreator.receiveGeocodeError($(".input-object-origin"));
		Sourcemap.ObjectCreator.receiveGeocodeError($(".input-upcobject-origin"));
	});

	$("body").bind("saveObjectConfirmation", function(e, data) { 
		Sourcemap.ObjectCreator.saveConfirm(data);		
	});
	
	$("ul#create-tabs li").click(Sourcemap.ObjectCreator.switchTab);
  },
  
  receiveGeocodeError: function(element) {
     $(element).qtip({ content: "We couldn't find this location...", show: { when: false, ready: true }, hide: 'click', position: { corner: { target: 'topMiddle', tooltip: 'bottomMiddle' } }, style: { width:340, 'font-size': 16,
	 tip: 'bottomMiddle', border: { width: 3, radius: 8, color: 'red' }, background: 'red', 'font-weight': 'bold',
	 color: 'white'
	 } });
	this.savingopen = true;
	this.savingrequested = false;
	setTimeout(function(){  $(".qtip").fadeOut("normal", function() {$(this).remove(); }); }, 2000);
  },

  receiveGeocodeMessage: function(element, address) {	
	$(element).val(address);
    $(element).qtip({ content: "We think you meant " + address + ".", show: { when: false, ready: true }, hide: 'click', position: { corner: { target: 'topMiddle', tooltip: 'bottomMiddle' } }, style: { width:340, 'font-size': 16,
	tip: 'bottomMiddle', border: { width: 3, radius: 8, color: '#f9e98e' }, background: '#fbf7aa', 'font-weight': 'bold',
	color: '#000'
	} });

	setTimeout(function(){  $(".qtip").fadeOut("normal", function() {$(this).remove(); }); }, 2000);
	if(this.savingopen & this.savingrequested) { this.savingopen = this.savingrequested = false; objectConfig.save();}	
  },

  save: function() {
	if($("#manual").hasClass("selected")) { //"Normal" method for saving objects
		//Make sure data has been entered properly
		if($(".input-object-name").val() == "" || $(".input-object-origin").val() == "") {
			Sourcemap.debug("Missing Name or Origin Information...");
		}		
		else {			
			this.savingrequested = true;			
			$(".input-object-name").change();
			$(".input-object-description").change();
			$(".input-upcobject-upc").change();
			$(".input-object-origin").change();
		}
	}
	else if($("#gbase").hasClass("selected")) { //Google Base-enhanced method
		//Provide a name if the user doesn't
		if($(".input-upcobject-name").val() == "" || $(".input-upcobject-name").val() == "(optional)")
			$(".input-upcobject-name").val($("#pname").text());
		//Get the image link from our preview image
		objectConfig.setImageLink($("#pimage").attr('src'));
		
		//Make sure data has been entered properly
		if($(".input-upcobject-name").val() == "" || $(".input-upcobject-origin").val() == "") {
			Sourcemap.debug("Missing Name or Origin Information...");
		}
		
		else {			
			this.savingrequested = true;
			$(".input-upcobject-name").change();
			$(".input-upcobject-description").change();
			$(".input-upcobject-upc").change();
			$(".input-upcobject-origin").change();
		}
	}
	
	else { //Import from excel method
		
	}

  },
	
  saveConfirm: function(slug) {
	Sourcemap.pageTracker._trackEvent("SMap", "Create", "Sourcemap "+objectConfig.type+"/"+slug+" created");
	window.location.href = Sourcemap.siteurl +objectConfig.type+'/' + slug + "#edit";
  },

  switchTab: function(tabid) {
		$("ul#create-tabs li").removeClass("selected");
		$(this).addClass("selected");
		
		$("#create-tab-content div.tab-content").addClass("hidden");
		$("#create-tab-content div#"+$(this).attr("id")+"-content").removeClass("hidden");
  }
};

$(document).ready(function() {	
	Sourcemap.ObjectCreator.init();
});
