/**
 * Sourcemap.Object: Representation of Sourcemap data.
 *
 * @version 0.8
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage js
 */

/**
 * Create an instance of Sourcemap
 * @see Sourcemap
 */
if ( typeof(Sourcemap) == "undefined" ) { Sourcemap = {}; }

Sourcemap.Object = function(obj) {
	jQuery.extend(this, obj);
	
	/**
	 * //===== Object Setter Functions =====
	 * @param val, this is the value of parameter to set the value
	 */
	this.setName = function(val) {
		this.name = Sourcemap.Util.utf8encode(val);
	};
	this.setDescription = function(val) {
		this.description = Sourcemap.Util.utf8encode(val);	   
		/**
		 *Trigger objectDescriptionUpdated in Editor.js 
		 */
		$("body").trigger("objectDescriptionUpdated", [val]);							
	};
	this.setImageLink = function(val) {
		this.imagelink = Sourcemap.Util.utf8encode(val);
	};
	this.setPermissions = function(val) {};
	this.setOrigin = function(val) {
		// TODO Object Depency on Smap existing
		SMap.geoCoder.getLocations(val, function(response){
				if(response.Status.code != 200) { 
					$("body").trigger("originGeoError", ["object"]);
					SMap.removeOrigin(); 
				} 
				else {
					if(typeof(response.Placemark[0]) != 'undefined') {
						place = response.Placemark[0];		
					    point = new GLatLng(place.Point.coordinates[1], place.Point.coordinates[0]);

						if(point != null && (point.lat() != 0 && point.lng() != 0) && response.Status.code == 200){						
							$("body").trigger("originGeoMessage", ["object", place.address]);	
							var adjPoint = point.lat() + "|" + point.lng();
							objectConfig.latlon = adjPoint;			
						
					       	SMap.setOrigin(val, objectConfig.parts); SMap.refreshMap(); //TODO objconfig dep
							objectConfig.origin = Sourcemap.Util.utf8encode(place.address); // TODO 
						}
						else { 
							$("body").trigger("originGeoError", ["object"]); 
							SMap.removeOrigin(); 
						}
					}
				}
				//TODO look at this
				for (var i=0;i<objectConfig.parts.length;i++) {
					objectConfig.setPartOrigin(i, objectConfig.parts[i].origin, false);
				}
			});
	};
	this.setDestination = function(val) {}; // TODO Remove?
	this.setVia = function(val) {}; // TODO Remove?	
	this.setWeight = function(val) { this.weight = val; };
	this.setShipping = function(val) { this.shipping = val; };
	this.setEmissions = function(val) { this.emissions = val; };
  	this.setEmbodiedEmissions = function(val) { this.embodied = val; };
  	this.setTransportEmissions = function(val) { this.transport = val; };
	this.setUPC = function(val) { this.upc = val; };

	this.setProcessEmissions = function(val) {this.processEmissions =val;};
	this.setEndoflifeEmissions = function(val) {this.endoflifeEmissions =val;};

	/**
	 * //===== Object Functions =====\\	
	 * call objects controller to save the part, process, endoflife
	 */
	this.save = function() {
	    var saveData = "data="+Sourcemap.Util.utf8encode(JSON.stringify(objectConfig));
		$("body").trigger("savingObject", [objectConfig]); 
	
		$.post(Sourcemap.siteurl+"objects/save",  saveData, this.saveConfirmation);
	};
	
   /**
	* check the parts flag, if it is "remove" then
	* set the parts value as null
	*/
	this.saveConfirmation = function(data) {
		for ( var i = 0; i < objectConfig.parts.length; i++ ) {
			if( typeof(objectConfig.parts[i]) != 'undefined') {
				if(objectConfig.parts[i].flag == "remove") {
					objectConfig.parts[i] = null;
				} else {
					objectConfig.parts[i].flag = "unchanged";
				}
			}
		}
		Sourcemap.pageTracker._trackEvent("SMap", "Save", "Sourcemap "+objectConfig.type+"-"+objectConfig.oid+" saved");				
		$("body").trigger("saveObjectConfirmation", [data]); 
	};
	
	/**
	 * 	//===== Part Functions =====\\	
	 * add a new part and set all the values
	 * @param part name, description, emissions, linkid
	 * trigger partAdded in Editor.js
	 */
	this.addPart = function(name, description, emissions, linkid) {
		// TODO partstub and partlocationstub depend		
		var id = this.partcount;

		var part = new Object();
		part.name = name != '' ? name : Sourcemap.Template.partstubname;
	  	part.description = typeof(description) != 'undefined' ? description : '';
	  	part.linkid = typeof(linkid) != 'undefined' ? linkid : 0;
	  	part.rawemissions = typeof(emissions) != 'undefined' ? Number(emissions) : 0;
	  	part.emissions = 0;

		part.flag = 'add';	
		part.imid = -1;
		part.weight = 0;
		part.shipping = 0;
		part.latlon = '0|0';	
		part.via = 'Part Via';
		part.transportemissions = 0;
		part.processemissions = 0;
		part.endoflifeemissions = 0;
		part.origin = Sourcemap.Template.partlocationstubname;
    
		this.parts[id] = part;
		this.partcount = this.partcount+1;
		Sourcemap.pageTracker._trackEvent("SMap", "AddedPart", "Added "+part.name+" to "+objectConfig.oid+"");				
		
		/**
		 * Trigger partAdded in Editor.js
		 */
		$("body").trigger("partAdded", [id, part.name, part.description, part.emissions, part.linkid]); 		
	};
	
    /**
	 * remove a part, remove the visualization, set the flag to remove
	 * @param id, this is the part id to be removed
	 * @see  #recalculateObjectValues
	 */
	this.removePart = function(id) {
		if(this.parts[id].flag == "add") {
			this.parts[id].flag = "unchanged";
		}
		else { this.parts[id].flag = "remove";}

		this.recalculateObjectValues();
		
		/**
	     * Trigger partRemoved in Editor.js
	     */
		$("body").trigger("partRemoved", [id]); 			
	};
	
	/**
	 * calculate totalemissions, transportEmissions, embodiedEmissions,
	 * endoflifeEmissions, processEmissions and set their values
	 * @see  #setEmissions
	 * @see  #setEmbodiedEmissions
	 * @see  #setTransportEmissions
	 * @see  #setShipping
	 * @see  #setWeight
	 * @see  #setProcessEmissions
	 * @see  #setEndoflifeEmissions
	 */
	this.recalculateObjectValues = function() {
		var emissionsTotal = 0;
		var shippingDistanceTotal = 0;
	        var weightTotal = 0;
		var transportEmissionsTotal = 0;
		var transportKGKM = 0;
		var embodiedEmissionsTotal = 0;
	        var processEmissionsTotal = 0;
	        var endoflifeEmissionsTotal = 0;

 		for ( var i = 0; i < this.parts.length; i++ ) {
 			var part = this.parts[i];
 			if(part.flag != "remove") {
 			    embodiedEmissionsTotal += parseFloat(part.emissions);
 			    transportEmissionsTotal += (parseFloat(part.shipping) * parseFloat(part.weight/1000) * parseFloat(part.transportemissions));
 			    weightTotal += parseFloat(part.weight);
			    processEmissionsTotal += Number(part.processemissions);				
				endoflifeEmissionsTotal += Number(part.endoflifeemissions);			    			    
 			    shippingDistanceTotal += parseFloat(part.shipping);					
 			    transportKGKM += (Number(part.weight) * Number(part.shipping));
 			}	
 		}
	    if(this.showembodied == "on") { emissionsTotal += embodiedEmissionsTotal;}		
	    if(this.showtransport == "on") { emissionsTotal += transportEmissionsTotal;}
	    if(this.showuse == "on" && this.usageemissions != null) { emissionsTotal += parseFloat(this.usageemissions);}
	    if(this.showprocess == "on") { emissionsTotal += processEmissionsTotal;}
        if(this.showendoflife == "on") {emissionsTotal += endoflifeEmissionsTotal; }
	    
	    this.setEmissions(emissionsTotal);
	    this.setEmbodiedEmissions(embodiedEmissionsTotal);
	    this.setTransportEmissions(transportEmissionsTotal);
	    this.setShipping(shippingDistanceTotal);
	    this.setWeight(weightTotal);
	    this.setProcessEmissions(processEmissionsTotal);
        this.setEndoflifeEmissions(endoflifeEmissionsTotal);

		/**
	     * Trigger objectTotalsCalculated in Editor.js
	     */
	    $("body").trigger("objectTotalsCalculated", [emissionsTotal, embodiedEmissionsTotal, transportEmissionsTotal, processEmissionsTotal, endoflifeEmissionsTotal, shippingDistanceTotal, weightTotal]);
	};

    /**
     * //===== Part Settor Functions =====\\
     * Set part name if there is a change in the part name
     * @param id, name, this is the part id and the name 
     */    
    this.setPartName = function(id, name) {	
		name = name.replace(/[']/g,'');	
		this.parts[id].name = name; 
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		    }
		/**
		 * Trigger partNameUpdated	  
		 */
		$("body").trigger("partNameUpdated", [id, name]);					
    };
    /**
     * Set part description to new description
     * @param id. description. This is the part_id, description
     */
	this.setPartDescription = function(id, description) {
			description = description.replace(/[']/g,'');
			this.parts[id].description = description; 				
			if(this.parts[id].flag != "add") {
				this.parts[id].flag = "changed";  
			}
			/**
			 * Trigger partDescriptionUpdated in Editor.js 
			 */
			$("body").trigger("partDescriptionUpdated", [id, description]);
	};
	/**
     * Set part weight to the new value
     * @param id, weight. This is the part id, new(entered) weight
     * @see  #recalculateObjectValues
     */
	this.setPartWeight = function(id, weight) {
		// TODO partkgfactor should e tied to something
		if(typeof(parseFloat(weight)) != "number") { weight = 0; }
		this.parts[id].weight = Number(weight*Sourcemap.Template.partkgfactor);				
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

		this.setPartEmissions(id, weight * this.parts[id].rawemissions, true);
		this.setPartTransportEmissions(id, $("#"+id+ " .input-via").val(), true); // TODO Strange Depend.	
		
		this.calculatePartProcessEmissions(id);
		this.calculatePartEndoflifeEmissions(id);		
		this.recalculateObjectValues(); $("body").trigger("partTotalsChanged", [id]); 
		/**
		 * Trigger partWeightUpdated in Editor.js
		 */
		$("body").trigger("partWeightUpdated", [id, weight]);
		
	};
	
	/**
     * Set the new part shipping
     * @param id, shipping. This is the part id and shipping
     * @see  #recalculateObjectValues
     */
	this.setPartShipping = function(id, shipping) {
		// TODO partkgfactor should e tied to something
		this.parts[id].shipping = Number(shipping);		
				
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

		this.recalculateObjectValues(); 
		
		/**
		 * Trigger partTotalsChanged, partShippingUpdated in Editor.js
		 */
		$("body").trigger("partTotalsChanged", [id]); 		
		$("body").trigger("partShippingUpdated", [id, shipping]);				
	};
	
	/**
     * Set set new part emissions
     * @param id, emissions, supress
     */
	this.setPartEmissions = function(id, emissions, suppress) {		
		this.parts[id].emissions = emissions; 
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

		if(!(suppress)) { this.recalculateObjectValues(); $("body").trigger("partTotalsChanged", [id]); }
	   /**
		* trigger partEmissionsUpdated
		*/
		$("body").trigger("partEmissionsUpdated", [id, emissions]);
	};
	/**
     * Set the part rawEmissions
     * @param id, rawemissions, supress
     */
	this.setPartRawEmissions = function(id, rawemissions, suppress) {	
		this.parts[id].rawemissions = rawemissions; 
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

		this.setPartEmissions(id, rawemissions * this.parts[id].weight, true);

		this.calculatePartEndoflifeEmissions(id);		
		if(!(suppress)) { this.recalculateObjectValues(); $("body").trigger("partTotalsChanged", [id]); }
	   /**
		* Trigger partEmissionsUpdated in Editor.js
		*/
		$("body").trigger("partRawEmissionsUpdated", [id, rawemissions]);
	};
	/**
	 * Set the paet transport emissions
	 * @param id, transportemissions, supress
	 */
	this.setPartTransportEmissions = function(id, transportemissions, suppress) {
		this.parts[id].transportemissions = transportemissions; 
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

		if(!(suppress)) { this.recalculateObjectValues(); $("body").trigger("partTotalsChanged", [id]); }

	   /**
		* Trigger partTransportEmissionsUpdated in Editor.js
		*/
		$("body").trigger("partTransportEmissionsUpdated", [id, transportemissions]);
	};
	/**
	 * Set the new part type
	 * @param id, via, type
	 * @see  #recalculateObjectValues
	 * @see  setPartTransportEmissions
	 */
	this.setPartVia = function(id, via, type) {
		this.parts[id].via = type;				
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

		this.setPartTransportEmissions(id, $("#"+id+ " .input-via").val(), true);		
		this.recalculateObjectValues(); $("body").trigger("partTotalsChanged", [id]); 

	   /**
		* Trigger partViaUpdated in Editor.js
		*/
		$("body").trigger("partViaUpdated", [id, via]);
	};
	/**
	 * Set the new part origin
	 * @param id, origin, geofeedback
	 */
	this.setPartOrigin = function(id, origin, geofeedback) {
		// Todo has objectConfig Depend. Callback should be seperated out
		objectOrigin = GLatLng.fromUrlValue(this.latlon.replace(/[|]/g,','));
		SMap.geoCoder.getLocations(Sourcemap.Util.utf8decode(origin), function(response){

			if(response.Status.code != 200) { if(geofeedback) { 
				$("body").trigger("originGeoError", [id]);
				SMap.removePart(objectConfig.parts[id], id); 
			} }

		    point = new GLatLng(response.Placemark[0].Point.coordinates[1], response.Placemark[0].Point.coordinates[0]);	
			place = response.Placemark[0];		
			
			if(point != null && (point.lat() != 0 && point.lng() != 0) && response.Status.code == 200){
				if(geofeedback) { $("body").trigger("originGeoMessage", [id, place.address]); }
				newDistance = Sourcemap.Util.getDistance(objectOrigin, point);
				if(objectConfig.parts[id].flag != "add") {
					objectConfig.parts[id].flag = "changed";  
				}
				objectConfig.parts[id].latlon = point.lat() + "|" + point.lng();	
				objectConfig.parts[id].origin = Sourcemap.Util.utf8encode(place.address);							
				objectConfig.setPartShipping(id, newDistance);				
				objectConfig.setPartTransportEmissions(id, $("#"+id+ " .input-via").val(), false);		
				// TODO needed? this.recalculateObjectValues()
				
				/**
				 * Set the new part origin
				 * @param id, origin, geofeedback
				 */
				$("body").trigger("partOriginUpdated", [id, place.address]);	
			}
			else { if(geofeedback) { 
				/**
				 * Tirgger originGeoError in Editor.js
				 */
				$("body").trigger("originGeoError", [id]);
				SMap.removePart(objectConfig.parts[id], id); 
			} }
		});
	};
	
    /**
     * //========Endoflife Setting Functions=====\\
     * @param partId, endoflfe_id, flag
     * @see  #recalculateObjectValues
     */
    this.addPartEndoflife = function(id, endid, endtype, emissions, unit, flag) {	
		if(typeof(this.parts[id].endoflife) == 'undefined') { this.parts[id].endoflife = [];}
		var endoflife = {
			type: endtype,
			emissions: emissions,
			unit: unit,
			flag: flag
		};
		this.parts[id].endoflife[endid] = endoflife;

		if(this.parts[id].flag != "add") { 
			this.parts[id].flag = "changed";
		}

		this.calculatePartEndoflifeEmissions(id);
		
		/**
		 * trigger partEndoflifeAdded in Editor.js
	 	 */
		$("body").trigger("partEndoflifeAdded", [id, endid, endtype, emissions, unit]);
    };

    this.removePartEndoflife = function(id, endid, flag){
        this.parts[id].endoflife[endid].flag = flag;
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";
		}
		this.calculatePartEndoflifeEmissions(id);
		$("body").trigger("partEndoflifeRemoved", [id, endid]); 				
    };

	/**
     * Set the endoflife type if there ia a change in type
     * @param PartId, endoflife_id, type
     * @see  #recalculateObjectValues
     */
    this.setEndoflifeType = function(id, endid, endtype) {
		this.parts[id].endoflife[endid].type = endtype;
		if(this.parts[id].flag != "add") { 
			this.parts[id].flag = "changed";
		}
	
		this.calculatePartEndoflifeEmissions(id);
		
		/**
		 * trigger partEndoflifeTypeUpdated in Editor.js 
		 */
		$("body").trigger("partEndoflifeTypeUpdated", [id, endid, type]);
    };

    /**
     * Set the endoflife emissions if there is a change in emissions
     * @param partId, endoflife_id, emissions
     * @see  #recalculateObjectValues
     */
	this.setPartEndoflifeEmissions = function(id, endid, emissions) {	
		this.parts[id].endoflife[endid].emissions = emissions;
		if(this.parts[id].flag != "add") { 
			this.parts[id].flag = "changed";
		}

		this.calculatePartEndoflifeEmissions(id);
		/**
		 * Trigger partEndoflifeEmissionsUpdated in Editor.js 
		 */
		$("body").trigger("partEndoflifeEmissionsUpdated", [id, endid, emissions]); 
		$("body").trigger("partTotalsChanged", [id]);  
	};
    
	this.calculatePartEndoflifeEmissions = function(id) {
		var part = this.parts[id];
		emissions = 0;
	    for(var i = 0; i < part.endoflife.length; i++) {
			if(part.endoflife[i].flag == false || typeof(part.endoflife[i].flag) == "undefined") {
			    if (part.endoflife[i].type == "reuse") { 
					emissions = -1 * parseFloat(part.emissions);
				} 
				else {	
					emissions = parseFloat(part.endoflife[i].emissions) * part.weight;
				}
			}	
		}
		part.endoflifeemissions = emissions;
		this.recalculateObjectValues();
		$("body").trigger("partEndoflifeCalculated", [id, emissions]);  
		$("body").trigger("partTotalsChanged", [id]); 		
	};
	
	/**
	 *  //========Process setting functions=====\\
	 * Add a new process 
	 * @param part_id, process_id, process_name, emissions, unit, flag, factor, check
	 * @see  #recalculateObjectValues
	 */
	this.addPartProcess = function (id, processid, name, emissions, unit, flag, factor, check){
		if(typeof(this.parts[id].process) == 'undefined') { this.parts[id].process = [];}

		var process = {
			name: name,
			emissions: emissions,
			unit: unit,
			flag: flag,
			factor: factor,
			label: 0
		};
		// label means only the factor or emissions have changed.
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";
		}
		this.parts[id].process[processid] = process;
		this.calculatePartProcessEmissions(id);
	    /**
	     *  trigger partProcessAdded in Editor.js 
	     */
		$("body").trigger("partProcessAdded", [id, processid, name, emissions, unit, flag, factor, check]);
	};
	
	/**
	 *  Remove a process
	 *  @param part_id, process_id, flag
	 *  @see  #recalculateObjectValues
	 */
	this.removePartProcess = function(id, processid, flag) {
		this.parts[id].process[processid].flag = flag;
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";
		}
		this.calculatePartProcessEmissions(id);
		/**
	     * Trigger partProcessRemoved in Editor.js 
	     */
	    $("body").trigger("partProcessRemoved", [id, processid]); 			
	};

	/**
	 * Set the process name if there is a change in the name
	 * @param part_id, process_id, process_name
	 */    	   
	this.setPartProcessName = function(id, processid, name){
		this.parts[id].process[processid].name = name;
		this.parts[id].process[processid].flag = 0;

		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

	    /**
	     * Trigger partProcessNameUpdated in Editor.js
	     */
		$("body").trigger("partProcessNameUpdated", [id, processid, name]);
	};
    
    /**
	 * Set the process unit if there is a change in the unit
	 * @param part_id, process_id, unit
	 * @see  #recalculateObjectValues
	 */
	this.setPartProcessUnit = function (id, processid, unit) {
		this.parts[id].process[processid].unit = unit;
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";  
		}

		this.calculatePartProcessEmissions(id);
		/**
	     * Trigger partProcessUnitUpdated in Editor.js
	     */
		$("body").trigger("partProcessUnitUpdated", [id, processid, unit]);
	};
  
	/**
	 * Set the process factor if there is a change in the factor
	 * @param part_id, process_id, factor, check
	 * @see  #recalculateObjectVaiues
	 */
	this.setPartProcessFactor = function(id, processid, factor, check){
		this.parts[id].process[processid].factor = factor;
		this.parts[id].process[process_id].label = 1;
		this.parts[id].process[process_id].flag = 0;
		if(this.parts[part_id].flag != "add") { this.parts[part_id].flag = "changed";}

		this.calculatePartProcessEmissions(id);
		/**
	     * Trigger partProcessFactorUpdated in Editor.js
	     */
		$("body").trigger("partProcessFactorUpdated", [id, processid, factor, check]);
	};
    
	/**
	 * Set the process emissions if there is a change in the emissions
	 * @param part_id, process_id, emissions
	 * @see  #recaiculateObjectValues
	 */
	this.setPartProcessEmissions = function(id, processid, emissions){
		this.parts[id].process[processid].emissions = emissions;		 
		this.parts[id].process[processid].label = 1;
		this.parts[id].process[processid].flag = 0;
		if(this.parts[id].flag != "add") {
			this.parts[id].flag = "changed";
		}
		this.calculatePartProcessEmissions(id);

	    /**
	     * Trigger partProcessEmissionsUpdated in Editor.js
	     */
		$("body").trigger("partProcessEmissionsUpdated", [id, processid, emissions]);
	};
   
	this.calculatePartProcessEmissions = function(id) {
		var part = this.parts[id];		
		var emissions = 0;	
		for(var i = 0; i < part.process.length; i++) {	
			//flag == false means add or existing
			// flag == true means flagged for deletion			
					
			if(part.process[i].flag == false || typeof(part.process[i].flag) == "undefined") {
				
				if(part.process[i].unit != "kg"){ 
					emissions += parseFloat(part.process[i].emissions) * parseFloat(part.process[i].factor);
				} 
				else {
					emissions += parseFloat(part.process[i].emissions) * part.weight;							   
				}  
			}		    
		}
		part.processemissions = emissions;
		this.recalculateObjectValues();
		$("body").trigger("partProcessCalculated", [id, emissions]);  		
		$("body").trigger("partTotalsChanged", [id]); 		
				
	};
	//===== Sample Debug Bindings =====\\
	$("body").bind("originGeoMessage", function(e, id, msg) { 
		Sourcemap.debug("Event::originGeoMessage {"+id+":"+msg+"}");
	});	
	$("body").bind("originGeoError", function(e, id) {
		Sourcemap.debug("Event::originGeoError:"+id);
	});
	$("body").bind("partAdded", function(e, id, name, desc, emiss, link) { 
		Sourcemap.debug("Event::partAdded {"+name+","+id+","+desc+","+emiss+","+link+"}");
	});
	$("body").bind("savingObject", function(e, obj) {
		Sourcemap.debug("Event::savingObject {"); Sourcemap.debug(obj); Sourcemap.debug("}");
	});	
	$("body").bind("saveObjectConfirmation", function(e, data) { 
		Sourcemap.debug("Event::saveObjectConfirmation {"+data+"}");
	});
	$("body").bind("objectTotalsCalculated", function(e, emis, embod, trans, process, end, ship, weight) {
		Sourcemap.debug("Event::objectTotalsCalculated {"+emis+","+embod+","+trans+","+process+","+end+","+ship+","+weight+"}");
	});
	
};

