//A modified version of openlayers_map that is designed to show
//multiple objects, but not their parts. Might need refactoring back into
//openlayers_map in the future.

Sourcemap.OpenMultiSourcemap = function (mapDiv, config, otherObjects, options) {
	this.map = null;
	this.popup = null;
	this.features = null;
	this.selectControl = null;
	this.partsExpander = null;
	this.selectedMarker = null;
	this.objectarray = config;
	this.options = options || {};
	this.parts = [];
	this.lines = [];
	this.objects = [];
	this.lastHash = null;
	
	//Colors
	var lightGreen = '008000'; //Parts, lines
	var green = '004000'; //Objects on this page
	var gray = '606060'; //Objects on other pages
	
	this.buildPartMarker = function(part, color, owner) {
		var markerLocation = null;
		var coords = [];
		var symbol = "*";
		var html = "";
		//We only want to map objects that have a specified lat/lon
		if ( part.latlon && part.latlon != "0|0") {
			coords = part.latlon.split(/\|/);

			markerLocation = new OpenLayers.LonLat( coords[1], coords[0] ).transform( this.map.displayProjection, this.map.projection );
			if(owner == null) {
				owner = "*";
			}
			html = 	"<strong>"+"("+owner+")</strong> " + part.name; //+ ":<br/>" + part.description;
			//Check for collisions with other markers
			for ( var i = 0; i < this.objects.length; i++ ) {
				if ( this.objects[i].isActive && this.objects[i].geometry.intersects( 
					new OpenLayers.Geometry.Point( markerLocation.lon, markerLocation.lat ) ) ) {
					//In the case of a collision, we'll absorb that marker's content into ours, and remove it from
					//the list of markers to be drawn.
					html += "<br/>" + this.objects[i].html;
					this.objects.splice( i, 1 );
					symbol = "...";
					color = green;
				}
			}
			
			var marker = this.makeMarker(markerLocation, symbol, color, html);
			marker.html = html;
			marker.isActive = true;
			marker.parts = null;
			this.parts.push(marker);
		}
		return markerLocation;
	};
	
	//Expands a feature's parts
	this.expandParts = function(feature) {
		this.modifyURL(feature.slug);
		this.clearParts();
		if(feature.parts != null) {
			var partLocation = null;
			for(i in feature.parts) {
				partLocation = this.buildPartMarker(feature.parts[i], lightGreen, feature.number);
				if(partLocation != null) {
					this.lines.push(this.dsAddLine(feature.location, partLocation, lightGreen));
				}
			}
		}
		this.features.addFeatures(this.lines);
		this.features.addFeatures(this.objects);
		this.features.addFeatures(this.parts);
	};
	
	//Clears all parts and part lines from the map
	this.clearParts = function() {
		this.features.removeFeatures(this.lines);
		this.features.removeFeatures(this.parts);
		
		this.parts = [];
		this.lines = [];
	};
	
	this.expandObjectFromURL = function() {
		if(window.location.hash && this.lastHash != window.location.hash) {
			this.lastHash = window.location.hash;
			//Extract the slug from the hash portion of the URL
			var slug = this.lastHash.substring(this.lastHash.indexOf('#')+1, this.lastHash.length);
			var index = null;
			//Find the index at which the slug from the URL matches an object's slug.
			//Maybe there's built-in JS for this?
			for(i in this.objects) {
				if(this.objects[i].slug == slug) {
					index = i;
					break;
				}
			}
			if(index != null) {
				if(this.objects[index].isActive) {
					this.expandParts(this.objects[index]);
				} else {
					var newMarker = this.makeMarker(this.objects[index].location, "+", green, this.objects[index].html);
					newMarker.slug = this.objects[index].slug;
					newMarker.number = null;
					newMarker.html = this.objects[index].html;
					newMarker.isActive = true;
					newMarker.parts = this.objects[index].parts;
					newMarker.location = this.objects[index].location;
					this.objects[index] = newMarker;
					this.expandParts(this.objects[index]);
				}
			}
		}
	}
	
	this.modifyURL = function(hash) {
		window.location.hash = hash;
	}
	
	this.createMap = function (mapName) {
		var mapOptions = {
			projection: new OpenLayers.Projection( "EPSG:900913" ),
			displayProjection: new OpenLayers.Projection( "EPSG:4326" ),
			maxExtent: new OpenLayers.Bounds(-20037508.34, -20037508.34,
						                             20037508.34, 20037508.34),
			theme:Sourcemap.baseurl + 'assets/scripts/OpenLayers-2.8/theme/default/style.css'
		};

		var nav = new OpenLayers.Control.Navigation();
		
		// keep this clicked point stationary, but increase the zoom
		nav.defaultDblClick = function(evt) {
			var clickPt = this.map.getLonLatFromViewPortPx( evt.xy );
			var centerPt = this.map.getCenter();
			
			centerPt.lon = (clickPt.lon + centerPt.lon)/2.0;
			centerPt.lat = (clickPt.lat + centerPt.lat)/2.0;			
			this.map.setCenter( centerPt, this.map.zoom + 1 );
		};
		
		mapOptions.controls = [ 
			nav
		];
	
		if (typeof(this.options.navPosition) != 'undefined') {
			if (this.options.navPosition != null) {
				mapOptions.controls.push( new OpenLayers.Control.PanZoom({ 
					position: this.options.navPosition 
				}) );
			}
		}
		else {
			mapOptions.controls.push( new OpenLayers.Control.PanZoom() );
		}		 			

		if(Sourcemap.tileprovider == "googlemaps") { 			
			var glayer = new OpenLayers.Layer.Google( "Base Map", {
				sphericalMercator:true,
				type: G_PHYSICAL_MAP,
				wrapDateLine: true
			} );
		}
		else if(Sourcemap.tileprovider == "cloudmade"){
			var glayer = new OpenLayers.Layer.CloudMade("CloudMade", {
			    key: '3bb82639a274433fb30120279ea08c76',
			    styleId: 4993
			});
			// BC9A493B41014CAABB98F0471D759707 
		}
		this.features = new OpenLayers.Layer.Vector( "Objects & Parts", {
			projection: mapOptions.projection,
			displayProjection: mapOptions.displayProjection,
			wrapDateLine: true
		});
	
		this.map = new OpenLayers.Map( mapName, mapOptions );
		this.selectControl = new OpenLayers.Control.SelectFeature( this.features,
			{ onSelect: this.markerSelect, onUnselect: this.markerUnselect, scope: this } );
		this.map.addControl( this.selectControl );
	
		this.selectControl.activate();
	
		this.map.addLayers( [glayer, this.features] );
		this.map.setCenter( new OpenLayers.LonLat(0,0), 2 );
		
		//Allows "back" and "forward" buttons to expand previous/next
		//object's parts. Small performance penalty, though. "that" and "this"
		//are a hack to make sure we're in the right scope when calling expandObjectFromURL.
		setInterval(function(that) { that.expandObjectFromURL(); }, 1000, this );
	};

	this.refreshMap = function(highlightedObjects, otherObjects) {
		this.selectControl.unselectAll( );

		this.features.removeFeatures( this.features.features );

		this.objects = [];
		this.parts = [];
		this.lines = [];
		this.loadMap(highlightedObjects, otherObjects);
	};
	
	this.buildObjectMarkers = function (objects, color, isActive) {
		var markerLocation = null;
		var firstMarkerLocation = null;
		var coords = [];
		var index = 1;
		var symbol = "+";
		var html = "";
		
		for(j in objects) {
			//We only want to map objects that have a specified lat/lon
			if ( objects[j].latlon && objects[j].latlon != "0|0") {
				coords = objects[j].latlon.split(/\|/);

				markerLocation = new OpenLayers.LonLat( coords[1], coords[0] ).transform( this.map.displayProjection, this.map.projection );
				
				if(isActive) {
					symbol = index + "";
				} else {
					symbol = "+";
				}
				
				html = 	"<strong>"+"["+symbol+"] "+"<a href='"+Sourcemap.baseurl+
								objects[j].type+"/"+objects[j].slug+"'>"+
								Sourcemap.Util.utf8decode(objects[j].name) + '</a></strong> Assembled in: ' + 
								Sourcemap.Util.utf8decode(objects[j].origin) + '<br/>' + 	Sourcemap.Util.utf8decode(objects[j].description);
				//Check for collisions with other markers
				for ( var i = 0; i < this.objects.length; i++ ) {
					if ( this.objects[i].isActive && this.objects[i].geometry.intersects( 
						new OpenLayers.Geometry.Point( markerLocation.lon, markerLocation.lat ) ) ) {
						//In the case of a collision, we'll absorb that marker's content into ours, and remove it from
						//the list of markers to be drawn.
						html += "<br/>" + this.objects[i].html;
						this.objects.splice( i, 1 );
						symbol = "...";
					}
				}
				
				if(firstMarkerLocation == null) {
					firstMarkerLocation = markerLocation;
				}
				var marker = this.makeMarker(markerLocation, symbol, color, html);
				marker.slug = objects[j].slug;
				marker.number = index;
				marker.html = html;
				marker.isActive = isActive;
				marker.parts = objects[j].parts;
				marker.location = markerLocation;
				this.objects.push(marker);
			}
			index++;
		}
		return firstMarkerLocation;
	};
	
	this.loadMap = function (highlightedObjects, otherObjects) {
		var mapCenter = null;
		if(otherObjects != []) {
			this.buildObjectMarkers(otherObjects, gray, false);
		}
		//This is called second in order to force the highlighted objects to the top
		mapCenter = this.buildObjectMarkers(highlightedObjects, green, true);
		this.features.addFeatures(this.objects);
		//Set the map's center as the first object created. Ideally, we would want some average
		//LonLat over all objects in the collection. Consider implementing in the future.
		this.map.setCenter(mapCenter, 2);
	};
	
	this.makeMarker = function(point, label, prcolor, info){
		var iconOptions = {
			label: label
		};
		if(label == "-") { // destination 
			iconOptions.primaryColor = prcolor;
			iconOptions.labelColor = "ffffff";
		}
		else if(label == "+") { // origin
			iconOptions.primaryColor = prcolor;
			iconOptions.labelColor = "ffffff";
		}
		else {
			iconOptions.primaryColor = prcolor;
			iconOptions.labelColor = "ffffff";
		}

		if(prcolor == "606060") {
			iconOptions.primaryColor = "003000";
			iconOptions.label = "";
			size=16;
		} else {
			size = 32;
		}
		// get a google chart for the map marker (same technique as MapIconMaker)
		if(prcolor == "606060") {
			var markerUrl = this.makeTinyMarkerImage( iconOptions.primaryColor, iconOptions.label, iconOptions.labelColor, size );
		} else {
			var markerUrl = this.makeMarkerImage( iconOptions.primaryColor, iconOptions.label, iconOptions.labelColor, size );
		}

		var marker = this.dsAddPoint(
			/* geometry */ new OpenLayers.Geometry.Point( point.lon, point.lat ), 
			/* attributes */ { label: label, info: info },
			/* style */ {
				externalGraphic: markerUrl,  
				graphicWidth: size,
				graphicHeight: size,
				graphicXOffset: -(size/2),
				graphicYOffset: -(size/2)
			}
		);

		return marker;
	};

	this.makeMarkerImage = function( primaryColor, label, labelColor, size ) {
		return 'http://chart.apis.google.com/chart?cht=it&chs=' + size + 'x' + size + '&chco=' + primaryColor + 
			',000000ff,ffffff01&chl=' + encodeURIComponent(label) + '&chx=' + labelColor + 
			',' + (size/2).toFixed() + '&chf=bg,s,00000000&ext=.png';
	};
	this.makeTinyMarkerImage = function( primaryColor, label, labelColor, size ) {
		return 'http://chart.apis.google.com/chart?cht=it&chs=' + size + 'x' + size + '&chco=' + primaryColor + 
			',000000ff,ffffff01&chl=' + encodeURIComponent(label) + '&chx=' + labelColor + 
			',' + (size/2).toFixed() + '&chf=bg,s,00000000|a,s,00000080&ext=.png';
	};
	this.dsAddPoint = function( geometry, attributes, style ) {
		var mpt = null;
		var globes = parseInt(((this.map.getExtent().getWidth() / this.map.getExtent().getHeight()) + 1).toFixed());

		var pts = new OpenLayers.Geometry.MultiPoint([geometry]);

		for ( var i = 1; i < globes; i++ ) {
			var lPt = new OpenLayers.Geometry.Point( geometry.x - i*CIRCUM, geometry.y );
			var rPt = new OpenLayers.Geometry.Point( geometry.x + i*CIRCUM, geometry.y );
			pts.addPoint( lPt );
			pts.addPoint( rPt );
		}

		return new OpenLayers.Feature.Vector( pts, attributes, style );
	};
	
	/**
	 * Make a polyline between two points with a specified color. This polyline
	 * function creates arcs that slowly curve over the map, attenuating to no arc
	 * along equal longitude. 
	 * @param {OpenLayers.LonLat} from One endpoint of the line.
	 * @param {OpenLayers.LonLat} to The other endpoint of the line.
	 * @return {OpenLayers.Geometry.LineString} A polyline.
	 */
	this.makePolyline = function(from, to){
		// create a 'bent' linestring
		var res = 8.0; // resolution
		var pts = new Array();
		var dx = to.lon - from.lon;
		var dy = to.lat - from.lat;
		var theta = (Math.PI / 2) - Math.atan( dy/dx );
		var maxdisp = Math.sqrt(dx*dx+dy*dy) * 0.05;

		if ( dx == 0 && dy == 0 ) {
			pts.push( new OpenLayers.Geometry.Point( from.lon, from.lat ) );
		}
		else {
			for ( var p = 0; p < res; p++ ) {
				var relamt = Math.sin(p/res*Math.PI) * maxdisp;
				if ( Math.abs(Math.sin(theta)) < Math.abs(Math.cos(theta)) ) {
					relamt *= Math.abs(Math.sin(Math.PI*dx/dy));
				}
				var ddx = Math.cos(theta+Math.PI) * relamt;
				var ddy = Math.sin(theta) * relamt;

				pts.push( new OpenLayers.Geometry.Point( from.lon + (dx*p/res) + ddx, from.lat + (dy*p/res) + ddy ) );
			}
		}
		pts.push( new OpenLayers.Geometry.Point( to.lon, to.lat ) );

		return new OpenLayers.Geometry.LineString(pts);
	};

	/**
	 * Add a line to the map in a Dateline Safe fashion. Dateline safe means the
	 * feature layer will replicate the specified features on either side of the 
	 * globe to prevent disappearing lines if the users scrolls over the date
	 * line.
	 * @param {OpenLayers.Geometry.Point} from An endpoint location.
	 * @param {OpenLayers.Geometry.Point} to An endpoint location.
	 * @param {String} color The color of the line.
	 * @return {OpenLayers.Feature.Vector}
	 */
	this.dsAddLine = function( from, to, color ) {
		var lines = new OpenLayers.Geometry.MultiLineString();
		color = ( color.indexOf('#') != 0 ) ? '#' + color : color;
		var globes = parseInt(((this.map.getExtent().getWidth() / this.map.getExtent().getHeight()) + 1).toFixed());            

		if ( Math.abs(to.lon - from.lon) > CIRCUM / 2 )
		{
			if ( to.lon > from.lon && from.lon + CIRCUM - to.lon < to.lon - from.lon ) {
				var fromp = new OpenLayers.LonLat( from.lon + CIRCUM, from.lat );
				lines.addComponents( [ this.makePolyline( fromp, to ) ] );
				
				for ( var i = 1; i < globes; i++ ) {
				  // connect to to from + CIRCUM
				  fromp = new OpenLayers.LonLat( from.lon - (i-1) * CIRCUM, from.lat );
				  var top = new OpenLayers.LonLat( to.lon - i * CIRCUM, to.lat );
				  lines.addComponents( [ this.makePolyline( fromp, top ) ] );

				  fromp = new OpenLayers.LonLat( from.lon + (i+1) * CIRCUM, from.lat );
				  top = new OpenLayers.LonLat( to.lon + i * CIRCUM, to.lat );
				  lines.addComponents( [ this.makePolyline( fromp, top ) ] );
				}
			}
			else {
				var top = new OpenLayers.LonLat( to.lon + CIRCUM, to.lat );
				lines.addComponents( [ this.makePolyline( from, top ) ] );
				
				for ( var i = 1; i < globes; i++ ) {
				  // connect to to from - CIRCUM
				  var fromp = new OpenLayers.LonLat( from.lon - i * CIRCUM, from.lat );
				  top = new OpenLayers.LonLat( to.lon - (i-1) * CIRCUM, to.lat );
				  lines.addComponents( [ this.makePolyline( fromp, top ) ] );

				  fromp = new OpenLayers.LonLat( from.lon + i * CIRCUM, from.lat );
				  top = new OpenLayers.LonLat( to.lon + (i+1) * CIRCUM, to.lat );
				  lines.addComponents( [ this.makePolyline( fromp, top ) ] );
				}
			}
		}
		else {
			lines.addComponents( [ this.makePolyline( from, to ) ] );
		
			for ( var i = 1; i < globes; i++ ) {
				var fromp = new OpenLayers.LonLat( from.lon - i * CIRCUM, from.lat );
				var top = new OpenLayers.LonLat( to.lon - i * CIRCUM, to.lat );
				lines.addComponents( [ this.makePolyline( fromp, top ) ] );

				fromp = new OpenLayers.LonLat( from.lon + i * CIRCUM, from.lat );
				top = new OpenLayers.LonLat( to.lon + i * CIRCUM, to.lat );
				lines.addComponents( [ this.makePolyline( fromp, top ) ] );
			}
		}

		return new OpenLayers.Feature.Vector(
			/* geometry */ lines, 
			/* attributes */ {},
			/* style */ { 
				strokeWidth: 5.0,
				strokeOpacity: 0.8,
				strokeColor: color
			}
		);
	};
	/**
	 * A callback to process the selection of a marker.
	 * @param {OpenLayers.Feature.Vector} feature The feature that has been selected.
	 */
	this.markerSelect = function( feature ) {
		if( feature.geometry instanceof OpenLayers.Geometry.LineString || !feature.isActive )
			return false;
		if(feature.parts != null) {
			this.expandParts(feature);
		}
		this.selectedMarker = feature;
		$("body").trigger("partSelected", [feature.attributes.label]);
	
		if ( this.popup == null ) {
			this.popup = new OpenLayers.Popup.FramedCloud( 'Marker',
				feature.geometry.getBounds().getCenterLonLat(),
				null,
				'<div style="padding:8px;">' + feature.attributes.info + '</div>',
				null,
				true,
				this.selectControl.unselect( this.selectedMarker )
			);
		}
		else {
			this.popup.contentHTML = '<div style="padding:8px;">' + feature.attributes.info + '</div>';
			this.popup.lonlat = feature.geometry.getBounds().getCenterLonLat();
		}
			
		this.map.addPopup( this.popup );
		this.popup.show();

		this.map.events.register( 'moveend', this, this.mapMoved );
	
		var size1 = this.popup.contentDiv.firstChild.getBoundingClientRect();
		var size2 = this.popup.contentDiv.getBoundingClientRect();
	
		// if the first child of the popup content wants to be bigger than the available size
		if ( (size1.top - size1.bottom) != (size2.top - size2.bottom) ) {
			this.popup.contentDiv.style.overflowY = 'scroll';
		}
		
	};

	/**
	 * A callback to process the deselection of a map marker.
	 * @param {OpenLayers.Feature.Vector} feature The feature to deselect.
	 */
	this.markerUnselect = function( feature ) {
		if ( this.popup != null )
			this.map.removePopup( this.popup );
		this.map.events.unregister( 'moveend', this, this.mapMoved );
	};

	this.createMap( mapDiv );
	this.loadMap( config, otherObjects );
}

var CIRCUM = 20037508.34 * 2; 